from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class LicenseType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    STREAMING: _ClassVar[LicenseType]
    OFFLINE: _ClassVar[LicenseType]
    AUTOMATIC: _ClassVar[LicenseType]

class PlatformVerificationStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PLATFORM_UNVERIFIED: _ClassVar[PlatformVerificationStatus]
    PLATFORM_TAMPERED: _ClassVar[PlatformVerificationStatus]
    PLATFORM_SOFTWARE_VERIFIED: _ClassVar[PlatformVerificationStatus]
    PLATFORM_HARDWARE_VERIFIED: _ClassVar[PlatformVerificationStatus]
    PLATFORM_NO_VERIFICATION: _ClassVar[PlatformVerificationStatus]
    PLATFORM_SECURE_STORAGE_SOFTWARE_VERIFIED: _ClassVar[PlatformVerificationStatus]

class ProtocolVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    VERSION_2_0: _ClassVar[ProtocolVersion]
    VERSION_2_1: _ClassVar[ProtocolVersion]
    VERSION_2_2: _ClassVar[ProtocolVersion]

class HashAlgorithmProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    HASH_ALGORITHM_UNSPECIFIED: _ClassVar[HashAlgorithmProto]
    HASH_ALGORITHM_SHA_1: _ClassVar[HashAlgorithmProto]
    HASH_ALGORITHM_SHA_256: _ClassVar[HashAlgorithmProto]
    HASH_ALGORITHM_SHA_384: _ClassVar[HashAlgorithmProto]
STREAMING: LicenseType
OFFLINE: LicenseType
AUTOMATIC: LicenseType
PLATFORM_UNVERIFIED: PlatformVerificationStatus
PLATFORM_TAMPERED: PlatformVerificationStatus
PLATFORM_SOFTWARE_VERIFIED: PlatformVerificationStatus
PLATFORM_HARDWARE_VERIFIED: PlatformVerificationStatus
PLATFORM_NO_VERIFICATION: PlatformVerificationStatus
PLATFORM_SECURE_STORAGE_SOFTWARE_VERIFIED: PlatformVerificationStatus
VERSION_2_0: ProtocolVersion
VERSION_2_1: ProtocolVersion
VERSION_2_2: ProtocolVersion
HASH_ALGORITHM_UNSPECIFIED: HashAlgorithmProto
HASH_ALGORITHM_SHA_1: HashAlgorithmProto
HASH_ALGORITHM_SHA_256: HashAlgorithmProto
HASH_ALGORITHM_SHA_384: HashAlgorithmProto

class LicenseIdentification(_message.Message):
    __slots__ = ("request_id", "session_id", "purchase_id", "type", "version", "provider_session_token")
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    PURCHASE_ID_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_SESSION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    request_id: bytes
    session_id: bytes
    purchase_id: bytes
    type: LicenseType
    version: int
    provider_session_token: bytes
    def __init__(self, request_id: _Optional[bytes] = ..., session_id: _Optional[bytes] = ..., purchase_id: _Optional[bytes] = ..., type: _Optional[_Union[LicenseType, str]] = ..., version: _Optional[int] = ..., provider_session_token: _Optional[bytes] = ...) -> None: ...

class License(_message.Message):
    __slots__ = ("id", "policy", "key", "license_start_time", "remote_attestation_verified", "provider_client_token", "protection_scheme", "srm_requirement", "srm_update", "platform_verification_status", "group_ids")
    class Policy(_message.Message):
        __slots__ = ("can_play", "can_persist", "can_renew", "rental_duration_seconds", "playback_duration_seconds", "license_duration_seconds", "renewal_recovery_duration_seconds", "renewal_server_url", "renewal_delay_seconds", "renewal_retry_interval_seconds", "renew_with_usage", "always_include_client_id", "play_start_grace_period_seconds", "soft_enforce_playback_duration", "soft_enforce_rental_duration")
        CAN_PLAY_FIELD_NUMBER: _ClassVar[int]
        CAN_PERSIST_FIELD_NUMBER: _ClassVar[int]
        CAN_RENEW_FIELD_NUMBER: _ClassVar[int]
        RENTAL_DURATION_SECONDS_FIELD_NUMBER: _ClassVar[int]
        PLAYBACK_DURATION_SECONDS_FIELD_NUMBER: _ClassVar[int]
        LICENSE_DURATION_SECONDS_FIELD_NUMBER: _ClassVar[int]
        RENEWAL_RECOVERY_DURATION_SECONDS_FIELD_NUMBER: _ClassVar[int]
        RENEWAL_SERVER_URL_FIELD_NUMBER: _ClassVar[int]
        RENEWAL_DELAY_SECONDS_FIELD_NUMBER: _ClassVar[int]
        RENEWAL_RETRY_INTERVAL_SECONDS_FIELD_NUMBER: _ClassVar[int]
        RENEW_WITH_USAGE_FIELD_NUMBER: _ClassVar[int]
        ALWAYS_INCLUDE_CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
        PLAY_START_GRACE_PERIOD_SECONDS_FIELD_NUMBER: _ClassVar[int]
        SOFT_ENFORCE_PLAYBACK_DURATION_FIELD_NUMBER: _ClassVar[int]
        SOFT_ENFORCE_RENTAL_DURATION_FIELD_NUMBER: _ClassVar[int]
        can_play: bool
        can_persist: bool
        can_renew: bool
        rental_duration_seconds: int
        playback_duration_seconds: int
        license_duration_seconds: int
        renewal_recovery_duration_seconds: int
        renewal_server_url: str
        renewal_delay_seconds: int
        renewal_retry_interval_seconds: int
        renew_with_usage: bool
        always_include_client_id: bool
        play_start_grace_period_seconds: int
        soft_enforce_playback_duration: bool
        soft_enforce_rental_duration: bool
        def __init__(self, can_play: bool = ..., can_persist: bool = ..., can_renew: bool = ..., rental_duration_seconds: _Optional[int] = ..., playback_duration_seconds: _Optional[int] = ..., license_duration_seconds: _Optional[int] = ..., renewal_recovery_duration_seconds: _Optional[int] = ..., renewal_server_url: _Optional[str] = ..., renewal_delay_seconds: _Optional[int] = ..., renewal_retry_interval_seconds: _Optional[int] = ..., renew_with_usage: bool = ..., always_include_client_id: bool = ..., play_start_grace_period_seconds: _Optional[int] = ..., soft_enforce_playback_duration: bool = ..., soft_enforce_rental_duration: bool = ...) -> None: ...
    class KeyContainer(_message.Message):
        __slots__ = ("id", "iv", "key", "type", "level", "required_protection", "requested_protection", "key_control", "operator_session_key_permissions", "video_resolution_constraints", "anti_rollback_usage_table", "track_label")
        class KeyType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            SIGNING: _ClassVar[License.KeyContainer.KeyType]
            CONTENT: _ClassVar[License.KeyContainer.KeyType]
            KEY_CONTROL: _ClassVar[License.KeyContainer.KeyType]
            OPERATOR_SESSION: _ClassVar[License.KeyContainer.KeyType]
            ENTITLEMENT: _ClassVar[License.KeyContainer.KeyType]
            OEM_CONTENT: _ClassVar[License.KeyContainer.KeyType]
        SIGNING: License.KeyContainer.KeyType
        CONTENT: License.KeyContainer.KeyType
        KEY_CONTROL: License.KeyContainer.KeyType
        OPERATOR_SESSION: License.KeyContainer.KeyType
        ENTITLEMENT: License.KeyContainer.KeyType
        OEM_CONTENT: License.KeyContainer.KeyType
        class SecurityLevel(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            SW_SECURE_CRYPTO: _ClassVar[License.KeyContainer.SecurityLevel]
            SW_SECURE_DECODE: _ClassVar[License.KeyContainer.SecurityLevel]
            HW_SECURE_CRYPTO: _ClassVar[License.KeyContainer.SecurityLevel]
            HW_SECURE_DECODE: _ClassVar[License.KeyContainer.SecurityLevel]
            HW_SECURE_ALL: _ClassVar[License.KeyContainer.SecurityLevel]
        SW_SECURE_CRYPTO: License.KeyContainer.SecurityLevel
        SW_SECURE_DECODE: License.KeyContainer.SecurityLevel
        HW_SECURE_CRYPTO: License.KeyContainer.SecurityLevel
        HW_SECURE_DECODE: License.KeyContainer.SecurityLevel
        HW_SECURE_ALL: License.KeyContainer.SecurityLevel
        class KeyControl(_message.Message):
            __slots__ = ("key_control_block", "iv")
            KEY_CONTROL_BLOCK_FIELD_NUMBER: _ClassVar[int]
            IV_FIELD_NUMBER: _ClassVar[int]
            key_control_block: bytes
            iv: bytes
            def __init__(self, key_control_block: _Optional[bytes] = ..., iv: _Optional[bytes] = ...) -> None: ...
        class OutputProtection(_message.Message):
            __slots__ = ("hdcp", "cgms_flags", "hdcp_srm_rule", "disable_analog_output", "disable_digital_output")
            class HDCP(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
                __slots__ = ()
                HDCP_NONE: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
                HDCP_V1: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
                HDCP_V2: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
                HDCP_V2_1: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
                HDCP_V2_2: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
                HDCP_V2_3: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
                HDCP_NO_DIGITAL_OUTPUT: _ClassVar[License.KeyContainer.OutputProtection.HDCP]
            HDCP_NONE: License.KeyContainer.OutputProtection.HDCP
            HDCP_V1: License.KeyContainer.OutputProtection.HDCP
            HDCP_V2: License.KeyContainer.OutputProtection.HDCP
            HDCP_V2_1: License.KeyContainer.OutputProtection.HDCP
            HDCP_V2_2: License.KeyContainer.OutputProtection.HDCP
            HDCP_V2_3: License.KeyContainer.OutputProtection.HDCP
            HDCP_NO_DIGITAL_OUTPUT: License.KeyContainer.OutputProtection.HDCP
            class CGMS(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
                __slots__ = ()
                CGMS_NONE: _ClassVar[License.KeyContainer.OutputProtection.CGMS]
                COPY_FREE: _ClassVar[License.KeyContainer.OutputProtection.CGMS]
                COPY_ONCE: _ClassVar[License.KeyContainer.OutputProtection.CGMS]
                COPY_NEVER: _ClassVar[License.KeyContainer.OutputProtection.CGMS]
            CGMS_NONE: License.KeyContainer.OutputProtection.CGMS
            COPY_FREE: License.KeyContainer.OutputProtection.CGMS
            COPY_ONCE: License.KeyContainer.OutputProtection.CGMS
            COPY_NEVER: License.KeyContainer.OutputProtection.CGMS
            class HdcpSrmRule(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
                __slots__ = ()
                HDCP_SRM_RULE_NONE: _ClassVar[License.KeyContainer.OutputProtection.HdcpSrmRule]
                CURRENT_SRM: _ClassVar[License.KeyContainer.OutputProtection.HdcpSrmRule]
            HDCP_SRM_RULE_NONE: License.KeyContainer.OutputProtection.HdcpSrmRule
            CURRENT_SRM: License.KeyContainer.OutputProtection.HdcpSrmRule
            HDCP_FIELD_NUMBER: _ClassVar[int]
            CGMS_FLAGS_FIELD_NUMBER: _ClassVar[int]
            HDCP_SRM_RULE_FIELD_NUMBER: _ClassVar[int]
            DISABLE_ANALOG_OUTPUT_FIELD_NUMBER: _ClassVar[int]
            DISABLE_DIGITAL_OUTPUT_FIELD_NUMBER: _ClassVar[int]
            hdcp: License.KeyContainer.OutputProtection.HDCP
            cgms_flags: License.KeyContainer.OutputProtection.CGMS
            hdcp_srm_rule: License.KeyContainer.OutputProtection.HdcpSrmRule
            disable_analog_output: bool
            disable_digital_output: bool
            def __init__(self, hdcp: _Optional[_Union[License.KeyContainer.OutputProtection.HDCP, str]] = ..., cgms_flags: _Optional[_Union[License.KeyContainer.OutputProtection.CGMS, str]] = ..., hdcp_srm_rule: _Optional[_Union[License.KeyContainer.OutputProtection.HdcpSrmRule, str]] = ..., disable_analog_output: bool = ..., disable_digital_output: bool = ...) -> None: ...
        class VideoResolutionConstraint(_message.Message):
            __slots__ = ("min_resolution_pixels", "max_resolution_pixels", "required_protection")
            MIN_RESOLUTION_PIXELS_FIELD_NUMBER: _ClassVar[int]
            MAX_RESOLUTION_PIXELS_FIELD_NUMBER: _ClassVar[int]
            REQUIRED_PROTECTION_FIELD_NUMBER: _ClassVar[int]
            min_resolution_pixels: int
            max_resolution_pixels: int
            required_protection: License.KeyContainer.OutputProtection
            def __init__(self, min_resolution_pixels: _Optional[int] = ..., max_resolution_pixels: _Optional[int] = ..., required_protection: _Optional[_Union[License.KeyContainer.OutputProtection, _Mapping]] = ...) -> None: ...
        class OperatorSessionKeyPermissions(_message.Message):
            __slots__ = ("allow_encrypt", "allow_decrypt", "allow_sign", "allow_signature_verify")
            ALLOW_ENCRYPT_FIELD_NUMBER: _ClassVar[int]
            ALLOW_DECRYPT_FIELD_NUMBER: _ClassVar[int]
            ALLOW_SIGN_FIELD_NUMBER: _ClassVar[int]
            ALLOW_SIGNATURE_VERIFY_FIELD_NUMBER: _ClassVar[int]
            allow_encrypt: bool
            allow_decrypt: bool
            allow_sign: bool
            allow_signature_verify: bool
            def __init__(self, allow_encrypt: bool = ..., allow_decrypt: bool = ..., allow_sign: bool = ..., allow_signature_verify: bool = ...) -> None: ...
        ID_FIELD_NUMBER: _ClassVar[int]
        IV_FIELD_NUMBER: _ClassVar[int]
        KEY_FIELD_NUMBER: _ClassVar[int]
        TYPE_FIELD_NUMBER: _ClassVar[int]
        LEVEL_FIELD_NUMBER: _ClassVar[int]
        REQUIRED_PROTECTION_FIELD_NUMBER: _ClassVar[int]
        REQUESTED_PROTECTION_FIELD_NUMBER: _ClassVar[int]
        KEY_CONTROL_FIELD_NUMBER: _ClassVar[int]
        OPERATOR_SESSION_KEY_PERMISSIONS_FIELD_NUMBER: _ClassVar[int]
        VIDEO_RESOLUTION_CONSTRAINTS_FIELD_NUMBER: _ClassVar[int]
        ANTI_ROLLBACK_USAGE_TABLE_FIELD_NUMBER: _ClassVar[int]
        TRACK_LABEL_FIELD_NUMBER: _ClassVar[int]
        id: bytes
        iv: bytes
        key: bytes
        type: License.KeyContainer.KeyType
        level: License.KeyContainer.SecurityLevel
        required_protection: License.KeyContainer.OutputProtection
        requested_protection: License.KeyContainer.OutputProtection
        key_control: License.KeyContainer.KeyControl
        operator_session_key_permissions: License.KeyContainer.OperatorSessionKeyPermissions
        video_resolution_constraints: _containers.RepeatedCompositeFieldContainer[License.KeyContainer.VideoResolutionConstraint]
        anti_rollback_usage_table: bool
        track_label: str
        def __init__(self, id: _Optional[bytes] = ..., iv: _Optional[bytes] = ..., key: _Optional[bytes] = ..., type: _Optional[_Union[License.KeyContainer.KeyType, str]] = ..., level: _Optional[_Union[License.KeyContainer.SecurityLevel, str]] = ..., required_protection: _Optional[_Union[License.KeyContainer.OutputProtection, _Mapping]] = ..., requested_protection: _Optional[_Union[License.KeyContainer.OutputProtection, _Mapping]] = ..., key_control: _Optional[_Union[License.KeyContainer.KeyControl, _Mapping]] = ..., operator_session_key_permissions: _Optional[_Union[License.KeyContainer.OperatorSessionKeyPermissions, _Mapping]] = ..., video_resolution_constraints: _Optional[_Iterable[_Union[License.KeyContainer.VideoResolutionConstraint, _Mapping]]] = ..., anti_rollback_usage_table: bool = ..., track_label: _Optional[str] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    POLICY_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    LICENSE_START_TIME_FIELD_NUMBER: _ClassVar[int]
    REMOTE_ATTESTATION_VERIFIED_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_CLIENT_TOKEN_FIELD_NUMBER: _ClassVar[int]
    PROTECTION_SCHEME_FIELD_NUMBER: _ClassVar[int]
    SRM_REQUIREMENT_FIELD_NUMBER: _ClassVar[int]
    SRM_UPDATE_FIELD_NUMBER: _ClassVar[int]
    PLATFORM_VERIFICATION_STATUS_FIELD_NUMBER: _ClassVar[int]
    GROUP_IDS_FIELD_NUMBER: _ClassVar[int]
    id: LicenseIdentification
    policy: License.Policy
    key: _containers.RepeatedCompositeFieldContainer[License.KeyContainer]
    license_start_time: int
    remote_attestation_verified: bool
    provider_client_token: bytes
    protection_scheme: int
    srm_requirement: bytes
    srm_update: bytes
    platform_verification_status: PlatformVerificationStatus
    group_ids: _containers.RepeatedScalarFieldContainer[bytes]
    def __init__(self, id: _Optional[_Union[LicenseIdentification, _Mapping]] = ..., policy: _Optional[_Union[License.Policy, _Mapping]] = ..., key: _Optional[_Iterable[_Union[License.KeyContainer, _Mapping]]] = ..., license_start_time: _Optional[int] = ..., remote_attestation_verified: bool = ..., provider_client_token: _Optional[bytes] = ..., protection_scheme: _Optional[int] = ..., srm_requirement: _Optional[bytes] = ..., srm_update: _Optional[bytes] = ..., platform_verification_status: _Optional[_Union[PlatformVerificationStatus, str]] = ..., group_ids: _Optional[_Iterable[bytes]] = ...) -> None: ...

class LicenseRequest(_message.Message):
    __slots__ = ("client_id", "content_id", "type", "request_time", "key_control_nonce_deprecated", "protocol_version", "key_control_nonce", "encrypted_client_id")
    class RequestType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        NEW: _ClassVar[LicenseRequest.RequestType]
        RENEWAL: _ClassVar[LicenseRequest.RequestType]
        RELEASE: _ClassVar[LicenseRequest.RequestType]
    NEW: LicenseRequest.RequestType
    RENEWAL: LicenseRequest.RequestType
    RELEASE: LicenseRequest.RequestType
    class ContentIdentification(_message.Message):
        __slots__ = ("widevine_pssh_data", "webm_key_id", "existing_license", "init_data")
        class WidevinePsshData(_message.Message):
            __slots__ = ("pssh_data", "license_type", "request_id")
            PSSH_DATA_FIELD_NUMBER: _ClassVar[int]
            LICENSE_TYPE_FIELD_NUMBER: _ClassVar[int]
            REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
            pssh_data: _containers.RepeatedScalarFieldContainer[bytes]
            license_type: LicenseType
            request_id: bytes
            def __init__(self, pssh_data: _Optional[_Iterable[bytes]] = ..., license_type: _Optional[_Union[LicenseType, str]] = ..., request_id: _Optional[bytes] = ...) -> None: ...
        class WebmKeyId(_message.Message):
            __slots__ = ("header", "license_type", "request_id")
            HEADER_FIELD_NUMBER: _ClassVar[int]
            LICENSE_TYPE_FIELD_NUMBER: _ClassVar[int]
            REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
            header: bytes
            license_type: LicenseType
            request_id: bytes
            def __init__(self, header: _Optional[bytes] = ..., license_type: _Optional[_Union[LicenseType, str]] = ..., request_id: _Optional[bytes] = ...) -> None: ...
        class ExistingLicense(_message.Message):
            __slots__ = ("license_id", "seconds_since_started", "seconds_since_last_played", "session_usage_table_entry")
            LICENSE_ID_FIELD_NUMBER: _ClassVar[int]
            SECONDS_SINCE_STARTED_FIELD_NUMBER: _ClassVar[int]
            SECONDS_SINCE_LAST_PLAYED_FIELD_NUMBER: _ClassVar[int]
            SESSION_USAGE_TABLE_ENTRY_FIELD_NUMBER: _ClassVar[int]
            license_id: LicenseIdentification
            seconds_since_started: int
            seconds_since_last_played: int
            session_usage_table_entry: bytes
            def __init__(self, license_id: _Optional[_Union[LicenseIdentification, _Mapping]] = ..., seconds_since_started: _Optional[int] = ..., seconds_since_last_played: _Optional[int] = ..., session_usage_table_entry: _Optional[bytes] = ...) -> None: ...
        class InitData(_message.Message):
            __slots__ = ("init_data_type", "init_data", "license_type", "request_id")
            class InitDataType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
                __slots__ = ()
                CENC: _ClassVar[LicenseRequest.ContentIdentification.InitData.InitDataType]
                WEBM: _ClassVar[LicenseRequest.ContentIdentification.InitData.InitDataType]
            CENC: LicenseRequest.ContentIdentification.InitData.InitDataType
            WEBM: LicenseRequest.ContentIdentification.InitData.InitDataType
            INIT_DATA_TYPE_FIELD_NUMBER: _ClassVar[int]
            INIT_DATA_FIELD_NUMBER: _ClassVar[int]
            LICENSE_TYPE_FIELD_NUMBER: _ClassVar[int]
            REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
            init_data_type: LicenseRequest.ContentIdentification.InitData.InitDataType
            init_data: bytes
            license_type: LicenseType
            request_id: bytes
            def __init__(self, init_data_type: _Optional[_Union[LicenseRequest.ContentIdentification.InitData.InitDataType, str]] = ..., init_data: _Optional[bytes] = ..., license_type: _Optional[_Union[LicenseType, str]] = ..., request_id: _Optional[bytes] = ...) -> None: ...
        WIDEVINE_PSSH_DATA_FIELD_NUMBER: _ClassVar[int]
        WEBM_KEY_ID_FIELD_NUMBER: _ClassVar[int]
        EXISTING_LICENSE_FIELD_NUMBER: _ClassVar[int]
        INIT_DATA_FIELD_NUMBER: _ClassVar[int]
        widevine_pssh_data: LicenseRequest.ContentIdentification.WidevinePsshData
        webm_key_id: LicenseRequest.ContentIdentification.WebmKeyId
        existing_license: LicenseRequest.ContentIdentification.ExistingLicense
        init_data: LicenseRequest.ContentIdentification.InitData
        def __init__(self, widevine_pssh_data: _Optional[_Union[LicenseRequest.ContentIdentification.WidevinePsshData, _Mapping]] = ..., webm_key_id: _Optional[_Union[LicenseRequest.ContentIdentification.WebmKeyId, _Mapping]] = ..., existing_license: _Optional[_Union[LicenseRequest.ContentIdentification.ExistingLicense, _Mapping]] = ..., init_data: _Optional[_Union[LicenseRequest.ContentIdentification.InitData, _Mapping]] = ...) -> None: ...
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    CONTENT_ID_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    REQUEST_TIME_FIELD_NUMBER: _ClassVar[int]
    KEY_CONTROL_NONCE_DEPRECATED_FIELD_NUMBER: _ClassVar[int]
    PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
    KEY_CONTROL_NONCE_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    client_id: ClientIdentification
    content_id: LicenseRequest.ContentIdentification
    type: LicenseRequest.RequestType
    request_time: int
    key_control_nonce_deprecated: bytes
    protocol_version: ProtocolVersion
    key_control_nonce: int
    encrypted_client_id: EncryptedClientIdentification
    def __init__(self, client_id: _Optional[_Union[ClientIdentification, _Mapping]] = ..., content_id: _Optional[_Union[LicenseRequest.ContentIdentification, _Mapping]] = ..., type: _Optional[_Union[LicenseRequest.RequestType, str]] = ..., request_time: _Optional[int] = ..., key_control_nonce_deprecated: _Optional[bytes] = ..., protocol_version: _Optional[_Union[ProtocolVersion, str]] = ..., key_control_nonce: _Optional[int] = ..., encrypted_client_id: _Optional[_Union[EncryptedClientIdentification, _Mapping]] = ...) -> None: ...

class MetricData(_message.Message):
    __slots__ = ("stage_name", "metric_data")
    class MetricType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        LATENCY: _ClassVar[MetricData.MetricType]
        TIMESTAMP: _ClassVar[MetricData.MetricType]
    LATENCY: MetricData.MetricType
    TIMESTAMP: MetricData.MetricType
    class TypeValue(_message.Message):
        __slots__ = ("type", "value")
        TYPE_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        type: MetricData.MetricType
        value: int
        def __init__(self, type: _Optional[_Union[MetricData.MetricType, str]] = ..., value: _Optional[int] = ...) -> None: ...
    STAGE_NAME_FIELD_NUMBER: _ClassVar[int]
    METRIC_DATA_FIELD_NUMBER: _ClassVar[int]
    stage_name: str
    metric_data: _containers.RepeatedCompositeFieldContainer[MetricData.TypeValue]
    def __init__(self, stage_name: _Optional[str] = ..., metric_data: _Optional[_Iterable[_Union[MetricData.TypeValue, _Mapping]]] = ...) -> None: ...

class VersionInfo(_message.Message):
    __slots__ = ("license_sdk_version", "license_service_version")
    LICENSE_SDK_VERSION_FIELD_NUMBER: _ClassVar[int]
    LICENSE_SERVICE_VERSION_FIELD_NUMBER: _ClassVar[int]
    license_sdk_version: str
    license_service_version: str
    def __init__(self, license_sdk_version: _Optional[str] = ..., license_service_version: _Optional[str] = ...) -> None: ...

class SignedMessage(_message.Message):
    __slots__ = ("type", "msg", "signature", "session_key", "remote_attestation", "metric_data", "service_version_info", "session_key_type", "oemcrypto_core_message")
    class MessageType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        LICENSE_REQUEST: _ClassVar[SignedMessage.MessageType]
        LICENSE: _ClassVar[SignedMessage.MessageType]
        ERROR_RESPONSE: _ClassVar[SignedMessage.MessageType]
        SERVICE_CERTIFICATE_REQUEST: _ClassVar[SignedMessage.MessageType]
        SERVICE_CERTIFICATE: _ClassVar[SignedMessage.MessageType]
        SUB_LICENSE: _ClassVar[SignedMessage.MessageType]
        CAS_LICENSE_REQUEST: _ClassVar[SignedMessage.MessageType]
        CAS_LICENSE: _ClassVar[SignedMessage.MessageType]
        EXTERNAL_LICENSE_REQUEST: _ClassVar[SignedMessage.MessageType]
        EXTERNAL_LICENSE: _ClassVar[SignedMessage.MessageType]
    LICENSE_REQUEST: SignedMessage.MessageType
    LICENSE: SignedMessage.MessageType
    ERROR_RESPONSE: SignedMessage.MessageType
    SERVICE_CERTIFICATE_REQUEST: SignedMessage.MessageType
    SERVICE_CERTIFICATE: SignedMessage.MessageType
    SUB_LICENSE: SignedMessage.MessageType
    CAS_LICENSE_REQUEST: SignedMessage.MessageType
    CAS_LICENSE: SignedMessage.MessageType
    EXTERNAL_LICENSE_REQUEST: SignedMessage.MessageType
    EXTERNAL_LICENSE: SignedMessage.MessageType
    class SessionKeyType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNDEFINED: _ClassVar[SignedMessage.SessionKeyType]
        WRAPPED_AES_KEY: _ClassVar[SignedMessage.SessionKeyType]
        EPHERMERAL_ECC_PUBLIC_KEY: _ClassVar[SignedMessage.SessionKeyType]
    UNDEFINED: SignedMessage.SessionKeyType
    WRAPPED_AES_KEY: SignedMessage.SessionKeyType
    EPHERMERAL_ECC_PUBLIC_KEY: SignedMessage.SessionKeyType
    TYPE_FIELD_NUMBER: _ClassVar[int]
    MSG_FIELD_NUMBER: _ClassVar[int]
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    SESSION_KEY_FIELD_NUMBER: _ClassVar[int]
    REMOTE_ATTESTATION_FIELD_NUMBER: _ClassVar[int]
    METRIC_DATA_FIELD_NUMBER: _ClassVar[int]
    SERVICE_VERSION_INFO_FIELD_NUMBER: _ClassVar[int]
    SESSION_KEY_TYPE_FIELD_NUMBER: _ClassVar[int]
    OEMCRYPTO_CORE_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    type: SignedMessage.MessageType
    msg: bytes
    signature: bytes
    session_key: bytes
    remote_attestation: bytes
    metric_data: _containers.RepeatedCompositeFieldContainer[MetricData]
    service_version_info: VersionInfo
    session_key_type: SignedMessage.SessionKeyType
    oemcrypto_core_message: bytes
    def __init__(self, type: _Optional[_Union[SignedMessage.MessageType, str]] = ..., msg: _Optional[bytes] = ..., signature: _Optional[bytes] = ..., session_key: _Optional[bytes] = ..., remote_attestation: _Optional[bytes] = ..., metric_data: _Optional[_Iterable[_Union[MetricData, _Mapping]]] = ..., service_version_info: _Optional[_Union[VersionInfo, _Mapping]] = ..., session_key_type: _Optional[_Union[SignedMessage.SessionKeyType, str]] = ..., oemcrypto_core_message: _Optional[bytes] = ...) -> None: ...

class ClientIdentification(_message.Message):
    __slots__ = ("type", "token", "client_info", "provider_client_token", "license_counter", "client_capabilities", "vmp_data", "device_credentials")
    class TokenType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        KEYBOX: _ClassVar[ClientIdentification.TokenType]
        DRM_DEVICE_CERTIFICATE: _ClassVar[ClientIdentification.TokenType]
        REMOTE_ATTESTATION_CERTIFICATE: _ClassVar[ClientIdentification.TokenType]
        OEM_DEVICE_CERTIFICATE: _ClassVar[ClientIdentification.TokenType]
    KEYBOX: ClientIdentification.TokenType
    DRM_DEVICE_CERTIFICATE: ClientIdentification.TokenType
    REMOTE_ATTESTATION_CERTIFICATE: ClientIdentification.TokenType
    OEM_DEVICE_CERTIFICATE: ClientIdentification.TokenType
    class NameValue(_message.Message):
        __slots__ = ("name", "value")
        NAME_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        name: str
        value: str
        def __init__(self, name: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class ClientCapabilities(_message.Message):
        __slots__ = ("client_token", "session_token", "video_resolution_constraints", "max_hdcp_version", "oem_crypto_api_version", "anti_rollback_usage_table", "srm_version", "can_update_srm", "supported_certificate_key_type", "analog_output_capabilities", "can_disable_analog_output", "resource_rating_tier")
        class HdcpVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            HDCP_NONE: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
            HDCP_V1: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
            HDCP_V2: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
            HDCP_V2_1: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
            HDCP_V2_2: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
            HDCP_V2_3: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
            HDCP_NO_DIGITAL_OUTPUT: _ClassVar[ClientIdentification.ClientCapabilities.HdcpVersion]
        HDCP_NONE: ClientIdentification.ClientCapabilities.HdcpVersion
        HDCP_V1: ClientIdentification.ClientCapabilities.HdcpVersion
        HDCP_V2: ClientIdentification.ClientCapabilities.HdcpVersion
        HDCP_V2_1: ClientIdentification.ClientCapabilities.HdcpVersion
        HDCP_V2_2: ClientIdentification.ClientCapabilities.HdcpVersion
        HDCP_V2_3: ClientIdentification.ClientCapabilities.HdcpVersion
        HDCP_NO_DIGITAL_OUTPUT: ClientIdentification.ClientCapabilities.HdcpVersion
        class CertificateKeyType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            RSA_2048: _ClassVar[ClientIdentification.ClientCapabilities.CertificateKeyType]
            RSA_3072: _ClassVar[ClientIdentification.ClientCapabilities.CertificateKeyType]
            ECC_SECP256R1: _ClassVar[ClientIdentification.ClientCapabilities.CertificateKeyType]
            ECC_SECP384R1: _ClassVar[ClientIdentification.ClientCapabilities.CertificateKeyType]
            ECC_SECP521R1: _ClassVar[ClientIdentification.ClientCapabilities.CertificateKeyType]
        RSA_2048: ClientIdentification.ClientCapabilities.CertificateKeyType
        RSA_3072: ClientIdentification.ClientCapabilities.CertificateKeyType
        ECC_SECP256R1: ClientIdentification.ClientCapabilities.CertificateKeyType
        ECC_SECP384R1: ClientIdentification.ClientCapabilities.CertificateKeyType
        ECC_SECP521R1: ClientIdentification.ClientCapabilities.CertificateKeyType
        class AnalogOutputCapabilities(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            ANALOG_OUTPUT_UNKNOWN: _ClassVar[ClientIdentification.ClientCapabilities.AnalogOutputCapabilities]
            ANALOG_OUTPUT_NONE: _ClassVar[ClientIdentification.ClientCapabilities.AnalogOutputCapabilities]
            ANALOG_OUTPUT_SUPPORTED: _ClassVar[ClientIdentification.ClientCapabilities.AnalogOutputCapabilities]
            ANALOG_OUTPUT_SUPPORTS_CGMS_A: _ClassVar[ClientIdentification.ClientCapabilities.AnalogOutputCapabilities]
        ANALOG_OUTPUT_UNKNOWN: ClientIdentification.ClientCapabilities.AnalogOutputCapabilities
        ANALOG_OUTPUT_NONE: ClientIdentification.ClientCapabilities.AnalogOutputCapabilities
        ANALOG_OUTPUT_SUPPORTED: ClientIdentification.ClientCapabilities.AnalogOutputCapabilities
        ANALOG_OUTPUT_SUPPORTS_CGMS_A: ClientIdentification.ClientCapabilities.AnalogOutputCapabilities
        CLIENT_TOKEN_FIELD_NUMBER: _ClassVar[int]
        SESSION_TOKEN_FIELD_NUMBER: _ClassVar[int]
        VIDEO_RESOLUTION_CONSTRAINTS_FIELD_NUMBER: _ClassVar[int]
        MAX_HDCP_VERSION_FIELD_NUMBER: _ClassVar[int]
        OEM_CRYPTO_API_VERSION_FIELD_NUMBER: _ClassVar[int]
        ANTI_ROLLBACK_USAGE_TABLE_FIELD_NUMBER: _ClassVar[int]
        SRM_VERSION_FIELD_NUMBER: _ClassVar[int]
        CAN_UPDATE_SRM_FIELD_NUMBER: _ClassVar[int]
        SUPPORTED_CERTIFICATE_KEY_TYPE_FIELD_NUMBER: _ClassVar[int]
        ANALOG_OUTPUT_CAPABILITIES_FIELD_NUMBER: _ClassVar[int]
        CAN_DISABLE_ANALOG_OUTPUT_FIELD_NUMBER: _ClassVar[int]
        RESOURCE_RATING_TIER_FIELD_NUMBER: _ClassVar[int]
        client_token: bool
        session_token: bool
        video_resolution_constraints: bool
        max_hdcp_version: ClientIdentification.ClientCapabilities.HdcpVersion
        oem_crypto_api_version: int
        anti_rollback_usage_table: bool
        srm_version: int
        can_update_srm: bool
        supported_certificate_key_type: _containers.RepeatedScalarFieldContainer[ClientIdentification.ClientCapabilities.CertificateKeyType]
        analog_output_capabilities: ClientIdentification.ClientCapabilities.AnalogOutputCapabilities
        can_disable_analog_output: bool
        resource_rating_tier: int
        def __init__(self, client_token: bool = ..., session_token: bool = ..., video_resolution_constraints: bool = ..., max_hdcp_version: _Optional[_Union[ClientIdentification.ClientCapabilities.HdcpVersion, str]] = ..., oem_crypto_api_version: _Optional[int] = ..., anti_rollback_usage_table: bool = ..., srm_version: _Optional[int] = ..., can_update_srm: bool = ..., supported_certificate_key_type: _Optional[_Iterable[_Union[ClientIdentification.ClientCapabilities.CertificateKeyType, str]]] = ..., analog_output_capabilities: _Optional[_Union[ClientIdentification.ClientCapabilities.AnalogOutputCapabilities, str]] = ..., can_disable_analog_output: bool = ..., resource_rating_tier: _Optional[int] = ...) -> None: ...
    class ClientCredentials(_message.Message):
        __slots__ = ("type", "token")
        TYPE_FIELD_NUMBER: _ClassVar[int]
        TOKEN_FIELD_NUMBER: _ClassVar[int]
        type: ClientIdentification.TokenType
        token: bytes
        def __init__(self, type: _Optional[_Union[ClientIdentification.TokenType, str]] = ..., token: _Optional[bytes] = ...) -> None: ...
    TYPE_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    CLIENT_INFO_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_CLIENT_TOKEN_FIELD_NUMBER: _ClassVar[int]
    LICENSE_COUNTER_FIELD_NUMBER: _ClassVar[int]
    CLIENT_CAPABILITIES_FIELD_NUMBER: _ClassVar[int]
    VMP_DATA_FIELD_NUMBER: _ClassVar[int]
    DEVICE_CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
    type: ClientIdentification.TokenType
    token: bytes
    client_info: _containers.RepeatedCompositeFieldContainer[ClientIdentification.NameValue]
    provider_client_token: bytes
    license_counter: int
    client_capabilities: ClientIdentification.ClientCapabilities
    vmp_data: bytes
    device_credentials: _containers.RepeatedCompositeFieldContainer[ClientIdentification.ClientCredentials]
    def __init__(self, type: _Optional[_Union[ClientIdentification.TokenType, str]] = ..., token: _Optional[bytes] = ..., client_info: _Optional[_Iterable[_Union[ClientIdentification.NameValue, _Mapping]]] = ..., provider_client_token: _Optional[bytes] = ..., license_counter: _Optional[int] = ..., client_capabilities: _Optional[_Union[ClientIdentification.ClientCapabilities, _Mapping]] = ..., vmp_data: _Optional[bytes] = ..., device_credentials: _Optional[_Iterable[_Union[ClientIdentification.ClientCredentials, _Mapping]]] = ...) -> None: ...

class EncryptedClientIdentification(_message.Message):
    __slots__ = ("provider_id", "service_certificate_serial_number", "encrypted_client_id", "encrypted_client_id_iv", "encrypted_privacy_key")
    PROVIDER_ID_FIELD_NUMBER: _ClassVar[int]
    SERVICE_CERTIFICATE_SERIAL_NUMBER_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_CLIENT_ID_IV_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_PRIVACY_KEY_FIELD_NUMBER: _ClassVar[int]
    provider_id: str
    service_certificate_serial_number: bytes
    encrypted_client_id: bytes
    encrypted_client_id_iv: bytes
    encrypted_privacy_key: bytes
    def __init__(self, provider_id: _Optional[str] = ..., service_certificate_serial_number: _Optional[bytes] = ..., encrypted_client_id: _Optional[bytes] = ..., encrypted_client_id_iv: _Optional[bytes] = ..., encrypted_privacy_key: _Optional[bytes] = ...) -> None: ...

class DrmCertificate(_message.Message):
    __slots__ = ("type", "serial_number", "creation_time_seconds", "expiration_time_seconds", "public_key", "system_id", "test_device_deprecated", "provider_id", "service_types", "algorithm", "rot_id", "encryption_key")
    class Type(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        ROOT: _ClassVar[DrmCertificate.Type]
        DEVICE_MODEL: _ClassVar[DrmCertificate.Type]
        DEVICE: _ClassVar[DrmCertificate.Type]
        SERVICE: _ClassVar[DrmCertificate.Type]
        PROVISIONER: _ClassVar[DrmCertificate.Type]
    ROOT: DrmCertificate.Type
    DEVICE_MODEL: DrmCertificate.Type
    DEVICE: DrmCertificate.Type
    SERVICE: DrmCertificate.Type
    PROVISIONER: DrmCertificate.Type
    class ServiceType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNKNOWN_SERVICE_TYPE: _ClassVar[DrmCertificate.ServiceType]
        LICENSE_SERVER_SDK: _ClassVar[DrmCertificate.ServiceType]
        LICENSE_SERVER_PROXY_SDK: _ClassVar[DrmCertificate.ServiceType]
        PROVISIONING_SDK: _ClassVar[DrmCertificate.ServiceType]
        CAS_PROXY_SDK: _ClassVar[DrmCertificate.ServiceType]
    UNKNOWN_SERVICE_TYPE: DrmCertificate.ServiceType
    LICENSE_SERVER_SDK: DrmCertificate.ServiceType
    LICENSE_SERVER_PROXY_SDK: DrmCertificate.ServiceType
    PROVISIONING_SDK: DrmCertificate.ServiceType
    CAS_PROXY_SDK: DrmCertificate.ServiceType
    class Algorithm(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNKNOWN_ALGORITHM: _ClassVar[DrmCertificate.Algorithm]
        RSA: _ClassVar[DrmCertificate.Algorithm]
        ECC_SECP256R1: _ClassVar[DrmCertificate.Algorithm]
        ECC_SECP384R1: _ClassVar[DrmCertificate.Algorithm]
        ECC_SECP521R1: _ClassVar[DrmCertificate.Algorithm]
    UNKNOWN_ALGORITHM: DrmCertificate.Algorithm
    RSA: DrmCertificate.Algorithm
    ECC_SECP256R1: DrmCertificate.Algorithm
    ECC_SECP384R1: DrmCertificate.Algorithm
    ECC_SECP521R1: DrmCertificate.Algorithm
    class EncryptionKey(_message.Message):
        __slots__ = ("public_key", "algorithm")
        PUBLIC_KEY_FIELD_NUMBER: _ClassVar[int]
        ALGORITHM_FIELD_NUMBER: _ClassVar[int]
        public_key: bytes
        algorithm: DrmCertificate.Algorithm
        def __init__(self, public_key: _Optional[bytes] = ..., algorithm: _Optional[_Union[DrmCertificate.Algorithm, str]] = ...) -> None: ...
    TYPE_FIELD_NUMBER: _ClassVar[int]
    SERIAL_NUMBER_FIELD_NUMBER: _ClassVar[int]
    CREATION_TIME_SECONDS_FIELD_NUMBER: _ClassVar[int]
    EXPIRATION_TIME_SECONDS_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_KEY_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_ID_FIELD_NUMBER: _ClassVar[int]
    TEST_DEVICE_DEPRECATED_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_ID_FIELD_NUMBER: _ClassVar[int]
    SERVICE_TYPES_FIELD_NUMBER: _ClassVar[int]
    ALGORITHM_FIELD_NUMBER: _ClassVar[int]
    ROT_ID_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTION_KEY_FIELD_NUMBER: _ClassVar[int]
    type: DrmCertificate.Type
    serial_number: bytes
    creation_time_seconds: int
    expiration_time_seconds: int
    public_key: bytes
    system_id: int
    test_device_deprecated: bool
    provider_id: str
    service_types: _containers.RepeatedScalarFieldContainer[DrmCertificate.ServiceType]
    algorithm: DrmCertificate.Algorithm
    rot_id: bytes
    encryption_key: DrmCertificate.EncryptionKey
    def __init__(self, type: _Optional[_Union[DrmCertificate.Type, str]] = ..., serial_number: _Optional[bytes] = ..., creation_time_seconds: _Optional[int] = ..., expiration_time_seconds: _Optional[int] = ..., public_key: _Optional[bytes] = ..., system_id: _Optional[int] = ..., test_device_deprecated: bool = ..., provider_id: _Optional[str] = ..., service_types: _Optional[_Iterable[_Union[DrmCertificate.ServiceType, str]]] = ..., algorithm: _Optional[_Union[DrmCertificate.Algorithm, str]] = ..., rot_id: _Optional[bytes] = ..., encryption_key: _Optional[_Union[DrmCertificate.EncryptionKey, _Mapping]] = ...) -> None: ...

class SignedDrmCertificate(_message.Message):
    __slots__ = ("drm_certificate", "signature", "signer", "hash_algorithm")
    DRM_CERTIFICATE_FIELD_NUMBER: _ClassVar[int]
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    SIGNER_FIELD_NUMBER: _ClassVar[int]
    HASH_ALGORITHM_FIELD_NUMBER: _ClassVar[int]
    drm_certificate: bytes
    signature: bytes
    signer: SignedDrmCertificate
    hash_algorithm: HashAlgorithmProto
    def __init__(self, drm_certificate: _Optional[bytes] = ..., signature: _Optional[bytes] = ..., signer: _Optional[_Union[SignedDrmCertificate, _Mapping]] = ..., hash_algorithm: _Optional[_Union[HashAlgorithmProto, str]] = ...) -> None: ...

class WidevinePsshData(_message.Message):
    __slots__ = ("key_ids", "content_id", "crypto_period_index", "protection_scheme", "crypto_period_seconds", "type", "key_sequence", "group_ids", "entitled_keys", "video_feature", "algorithm", "provider", "track_type", "policy", "grouped_license")
    class Type(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        SINGLE: _ClassVar[WidevinePsshData.Type]
        ENTITLEMENT: _ClassVar[WidevinePsshData.Type]
        ENTITLED_KEY: _ClassVar[WidevinePsshData.Type]
    SINGLE: WidevinePsshData.Type
    ENTITLEMENT: WidevinePsshData.Type
    ENTITLED_KEY: WidevinePsshData.Type
    class Algorithm(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNENCRYPTED: _ClassVar[WidevinePsshData.Algorithm]
        AESCTR: _ClassVar[WidevinePsshData.Algorithm]
    UNENCRYPTED: WidevinePsshData.Algorithm
    AESCTR: WidevinePsshData.Algorithm
    class EntitledKey(_message.Message):
        __slots__ = ("entitlement_key_id", "key_id", "key", "iv", "entitlement_key_size_bytes")
        ENTITLEMENT_KEY_ID_FIELD_NUMBER: _ClassVar[int]
        KEY_ID_FIELD_NUMBER: _ClassVar[int]
        KEY_FIELD_NUMBER: _ClassVar[int]
        IV_FIELD_NUMBER: _ClassVar[int]
        ENTITLEMENT_KEY_SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
        entitlement_key_id: bytes
        key_id: bytes
        key: bytes
        iv: bytes
        entitlement_key_size_bytes: int
        def __init__(self, entitlement_key_id: _Optional[bytes] = ..., key_id: _Optional[bytes] = ..., key: _Optional[bytes] = ..., iv: _Optional[bytes] = ..., entitlement_key_size_bytes: _Optional[int] = ...) -> None: ...
    KEY_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTENT_ID_FIELD_NUMBER: _ClassVar[int]
    CRYPTO_PERIOD_INDEX_FIELD_NUMBER: _ClassVar[int]
    PROTECTION_SCHEME_FIELD_NUMBER: _ClassVar[int]
    CRYPTO_PERIOD_SECONDS_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    KEY_SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    GROUP_IDS_FIELD_NUMBER: _ClassVar[int]
    ENTITLED_KEYS_FIELD_NUMBER: _ClassVar[int]
    VIDEO_FEATURE_FIELD_NUMBER: _ClassVar[int]
    ALGORITHM_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_FIELD_NUMBER: _ClassVar[int]
    TRACK_TYPE_FIELD_NUMBER: _ClassVar[int]
    POLICY_FIELD_NUMBER: _ClassVar[int]
    GROUPED_LICENSE_FIELD_NUMBER: _ClassVar[int]
    key_ids: _containers.RepeatedScalarFieldContainer[bytes]
    content_id: bytes
    crypto_period_index: int
    protection_scheme: int
    crypto_period_seconds: int
    type: WidevinePsshData.Type
    key_sequence: int
    group_ids: _containers.RepeatedScalarFieldContainer[bytes]
    entitled_keys: _containers.RepeatedCompositeFieldContainer[WidevinePsshData.EntitledKey]
    video_feature: str
    algorithm: WidevinePsshData.Algorithm
    provider: str
    track_type: str
    policy: str
    grouped_license: bytes
    def __init__(self, key_ids: _Optional[_Iterable[bytes]] = ..., content_id: _Optional[bytes] = ..., crypto_period_index: _Optional[int] = ..., protection_scheme: _Optional[int] = ..., crypto_period_seconds: _Optional[int] = ..., type: _Optional[_Union[WidevinePsshData.Type, str]] = ..., key_sequence: _Optional[int] = ..., group_ids: _Optional[_Iterable[bytes]] = ..., entitled_keys: _Optional[_Iterable[_Union[WidevinePsshData.EntitledKey, _Mapping]]] = ..., video_feature: _Optional[str] = ..., algorithm: _Optional[_Union[WidevinePsshData.Algorithm, str]] = ..., provider: _Optional[str] = ..., track_type: _Optional[str] = ..., policy: _Optional[str] = ..., grouped_license: _Optional[bytes] = ...) -> None: ...

class VmpData(_message.Message):
    __slots__ = ("certificates", "signed_binary_info", "cdm_host_files", "cdm_call_chain_files", "current_process_file", "host_file_indexes", "call_chain_file_indexes", "current_process_file_index")
    class SignedBinaryInfo(_message.Message):
        __slots__ = ("file_name", "certificate_index", "binary_hash", "flags", "signature", "hash_algorithm")
        class HashAlgorithmProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
            __slots__ = ()
            HASH_ALGORITHM_UNSPECIFIED: _ClassVar[VmpData.SignedBinaryInfo.HashAlgorithmProto]
            HASH_ALGORITHM_SHA_1: _ClassVar[VmpData.SignedBinaryInfo.HashAlgorithmProto]
            HASH_ALGORITHM_SHA_256: _ClassVar[VmpData.SignedBinaryInfo.HashAlgorithmProto]
            HASH_ALGORITHM_SHA_384: _ClassVar[VmpData.SignedBinaryInfo.HashAlgorithmProto]
        HASH_ALGORITHM_UNSPECIFIED: VmpData.SignedBinaryInfo.HashAlgorithmProto
        HASH_ALGORITHM_SHA_1: VmpData.SignedBinaryInfo.HashAlgorithmProto
        HASH_ALGORITHM_SHA_256: VmpData.SignedBinaryInfo.HashAlgorithmProto
        HASH_ALGORITHM_SHA_384: VmpData.SignedBinaryInfo.HashAlgorithmProto
        FILE_NAME_FIELD_NUMBER: _ClassVar[int]
        CERTIFICATE_INDEX_FIELD_NUMBER: _ClassVar[int]
        BINARY_HASH_FIELD_NUMBER: _ClassVar[int]
        FLAGS_FIELD_NUMBER: _ClassVar[int]
        SIGNATURE_FIELD_NUMBER: _ClassVar[int]
        HASH_ALGORITHM_FIELD_NUMBER: _ClassVar[int]
        file_name: str
        certificate_index: int
        binary_hash: bytes
        flags: int
        signature: bytes
        hash_algorithm: VmpData.SignedBinaryInfo.HashAlgorithmProto
        def __init__(self, file_name: _Optional[str] = ..., certificate_index: _Optional[int] = ..., binary_hash: _Optional[bytes] = ..., flags: _Optional[int] = ..., signature: _Optional[bytes] = ..., hash_algorithm: _Optional[_Union[VmpData.SignedBinaryInfo.HashAlgorithmProto, str]] = ...) -> None: ...
    CERTIFICATES_FIELD_NUMBER: _ClassVar[int]
    SIGNED_BINARY_INFO_FIELD_NUMBER: _ClassVar[int]
    CDM_HOST_FILES_FIELD_NUMBER: _ClassVar[int]
    CDM_CALL_CHAIN_FILES_FIELD_NUMBER: _ClassVar[int]
    CURRENT_PROCESS_FILE_FIELD_NUMBER: _ClassVar[int]
    HOST_FILE_INDEXES_FIELD_NUMBER: _ClassVar[int]
    CALL_CHAIN_FILE_INDEXES_FIELD_NUMBER: _ClassVar[int]
    CURRENT_PROCESS_FILE_INDEX_FIELD_NUMBER: _ClassVar[int]
    certificates: _containers.RepeatedScalarFieldContainer[bytes]
    signed_binary_info: _containers.RepeatedCompositeFieldContainer[VmpData.SignedBinaryInfo]
    cdm_host_files: _containers.RepeatedCompositeFieldContainer[VmpData.SignedBinaryInfo]
    cdm_call_chain_files: _containers.RepeatedCompositeFieldContainer[VmpData.SignedBinaryInfo]
    current_process_file: VmpData.SignedBinaryInfo
    host_file_indexes: _containers.RepeatedScalarFieldContainer[int]
    call_chain_file_indexes: _containers.RepeatedScalarFieldContainer[int]
    current_process_file_index: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, certificates: _Optional[_Iterable[bytes]] = ..., signed_binary_info: _Optional[_Iterable[_Union[VmpData.SignedBinaryInfo, _Mapping]]] = ..., cdm_host_files: _Optional[_Iterable[_Union[VmpData.SignedBinaryInfo, _Mapping]]] = ..., cdm_call_chain_files: _Optional[_Iterable[_Union[VmpData.SignedBinaryInfo, _Mapping]]] = ..., current_process_file: _Optional[_Union[VmpData.SignedBinaryInfo, _Mapping]] = ..., host_file_indexes: _Optional[_Iterable[int]] = ..., call_chain_file_indexes: _Optional[_Iterable[int]] = ..., current_process_file_index: _Optional[_Iterable[int]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: license_protocol.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'license_protocol.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16license_protocol.proto\x12\x1bpywidevine_license_protocol\"\xbd\x01\n\x15LicenseIdentification\x12\x12\n\nrequest_id\x18\x01 \x01(\x0c\x12\x12\n\nsession_id\x18\x02 \x01(\x0c\x12\x13\n\x0bpurchase_id\x18\x03 \x01(\x0c\x12\x36\n\x04type\x18\x04 \x01(\x0e\x32(.pywidevine_license_protocol.LicenseType\x12\x0f\n\x07version\x18\x05 \x01(\x05\x12\x1e\n\x16provider_session_token\x18\x06 \x01(\x0c\"\xf1\x18\n\x07License\x12>\n\x02id\x18\x01 \x01(\x0b\x32\x32.pywidevine_license_protocol.LicenseIdentification\x12;\n\x06policy\x18\x02 \x01(\x0b\x32+.pywidevine_license_protocol.License.Policy\x12>\n\x03key\x18\x03 \x03(\x0b\x32\x31.pywidevine_license_protocol.License.KeyContainer\x12\x1a\n\x12license_start_time\x18\x04 \x01(\x03\x12*\n\x1bremote_attestation_verified\x18\x05 \x01(\x08:\x05\x66\x61lse\x12\x1d\n\x15provider_client_token\x18\x06 \x01(\x0c\x12\x19\n\x11protection_scheme\x18\x07 \x01(\r\x12\x17\n\x0fsrm_requirement\x18\x08 \x01(\x0c\x12\x12\n\nsrm_update\x18\t \x01(\x0c\x12w\n\x1cplatform_verification_status\x18\n \x01(\x0e\x32\x37.pywidevine_license_protocol.PlatformVerificationStatus:\x18PLATFORM_NO_VERIFICATION\x12\x11\n\tgroup_ids\x18\x0b \x03(\x0c\x1a\xae\x04\n\x06Policy\x12\x17\n\x08\x63\x61n_play\x18\x01 \x01(\x08:\x05\x66\x61lse\x12\x1a\n\x0b\x63\x61n_persist\x18\x02 \x01(\x08:\x05\x66\x61lse\x12\x18\n\tcan_renew\x18\x03 \x01(\x08:\x05\x66\x61lse\x12\"\n\x17rental_duration_seconds\x18\x04 \x01(\x03:\x01\x30\x12$\n\x19playback_duration_seconds\x18\x05 \x01(\x03:\x01\x30\x12#\n\x18license_duration_seconds\x18\x06 \x01(\x03:\x01\x30\x12,\n!renewal_recovery_duration_seconds\x18\x07 \x01(\x03:\x01\x30\x12\x1a\n\x12renewal_server_url\x18\x08 \x01(\t\x12 \n\x15renewal_delay_seconds\x18\t \x01(\x03:\x01\x30\x12)\n\x1erenewal_retry_interval_seconds\x18\n \x01(\x03:\x01\x30\x12\x1f\n\x10renew_with_usage\x18\x0b \x01(\x08:\x05\x66\x61lse\x12\'\n\x18\x61lways_include_client_id\x18\x0c \x01(\x08:\x05\x66\x61lse\x12*\n\x1fplay_start_grace_period_seconds\x18\r \x01(\x03:\x01\x30\x12-\n\x1esoft_enforce_playback_duration\x18\x0e \x01(\x08:\x05\x66\x61lse\x12*\n\x1csoft_enforce_rental_duration\x18\x0f \x01(\x08:\x04true\x1a\xbc\x10\n\x0cKeyContainer\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\n\n\x02iv\x18\x02 \x01(\x0c\x12\x0b\n\x03key\x18\x03 \x01(\x0c\x12G\n\x04type\x18\x04 \x01(\x0e\x32\x39.pywidevine_license_protocol.License.KeyContainer.KeyType\x12`\n\x05level\x18\x05 \x01(\x0e\x32?.pywidevine_license_protocol.License.KeyContainer.SecurityLevel:\x10SW_SECURE_CRYPTO\x12_\n\x13required_protection\x18\x06 \x01(\x0b\x32\x42.pywidevine_license_protocol.License.KeyContainer.OutputProtection\x12`\n\x14requested_protection\x18\x07 \x01(\x0b\x32\x42.pywidevine_license_protocol.License.KeyContainer.OutputProtection\x12Q\n\x0bkey_control\x18\x08 \x01(\x0b\x32<.pywidevine_license_protocol.License.KeyContainer.KeyControl\x12y\n operator_session_key_permissions\x18\t \x01(\x0b\x32O.pywidevine_license_protocol.License.KeyContainer.OperatorSessionKeyPermissions\x12q\n\x1cvideo_resolution_constraints\x18\n \x03(\x0b\x32K.pywidevine_license_protocol.License.KeyContainer.VideoResolutionConstraint\x12(\n\x19\x61nti_rollback_usage_table\x18\x0b \x01(\x08:\x05\x66\x61lse\x12\x13\n\x0btrack_label\x18\x0c \x01(\t\x1a\x33\n\nKeyControl\x12\x19\n\x11key_control_block\x18\x01 \x01(\x0c\x12\n\n\x02iv\x18\x02 \x01(\x0c\x1a\x9c\x05\n\x10OutputProtection\x12`\n\x04hdcp\x18\x01 \x01(\x0e\x32G.pywidevine_license_protocol.License.KeyContainer.OutputProtection.HDCP:\tHDCP_NONE\x12\x66\n\ncgms_flags\x18\x02 \x01(\x0e\x32G.pywidevine_license_protocol.License.KeyContainer.OutputProtection.CGMS:\tCGMS_NONE\x12y\n\rhdcp_srm_rule\x18\x03 \x01(\x0e\x32N.pywidevine_license_protocol.License.KeyContainer.OutputProtection.HdcpSrmRule:\x12HDCP_SRM_RULE_NONE\x12$\n\x15\x64isable_analog_output\x18\x04 \x01(\x08:\x05\x66\x61lse\x12%\n\x16\x64isable_digital_output\x18\x05 \x01(\x08:\x05\x66\x61lse\"y\n\x04HDCP\x12\r\n\tHDCP_NONE\x10\x00\x12\x0b\n\x07HDCP_V1\x10\x01\x12\x0b\n\x07HDCP_V2\x10\x02\x12\r\n\tHDCP_V2_1\x10\x03\x12\r\n\tHDCP_V2_2\x10\x04\x12\r\n\tHDCP_V2_3\x10\x05\x12\x1b\n\x16HDCP_NO_DIGITAL_OUTPUT\x10\xff\x01\"C\n\x04\x43GMS\x12\r\n\tCGMS_NONE\x10*\x12\r\n\tCOPY_FREE\x10\x00\x12\r\n\tCOPY_ONCE\x10\x02\x12\x0e\n\nCOPY_NEVER\x10\x03\"6\n\x0bHdcpSrmRule\x12\x16\n\x12HDCP_SRM_RULE_NONE\x10\x00\x12\x0f\n\x0b\x43URRENT_SRM\x10\x01\x1a\xba\x01\n\x19VideoResolutionConstraint\x12\x1d\n\x15min_resolution_pixels\x18\x01 \x01(\r\x12\x1d\n\x15max_resolution_pixels\x18\x02 \x01(\r\x12_\n\x13required_protection\x18\x03 \x01(\x0b\x32\x42.pywidevine_license_protocol.License.KeyContainer.OutputProtection\x1a\x9d\x01\n\x1dOperatorSessionKeyPermissions\x12\x1c\n\rallow_encrypt\x18\x01 \x01(\x08:\x05\x66\x61lse\x12\x1c\n\rallow_decrypt\x18\x02 \x01(\x08:\x05\x66\x61lse\x12\x19\n\nallow_sign\x18\x03 \x01(\x08:\x05\x66\x61lse\x12%\n\x16\x61llow_signature_verify\x18\x04 \x01(\x08:\x05\x66\x61lse\"l\n\x07KeyType\x12\x0b\n\x07SIGNING\x10\x01\x12\x0b\n\x07\x43ONTENT\x10\x02\x12\x0f\n\x0bKEY_CONTROL\x10\x03\x12\x14\n\x10OPERATOR_SESSION\x10\x04\x12\x0f\n\x0b\x45NTITLEMENT\x10\x05\x12\x0f\n\x0bOEM_CONTENT\x10\x06\"z\n\rSecurityLevel\x12\x14\n\x10SW_SECURE_CRYPTO\x10\x01\x12\x14\n\x10SW_SECURE_DECODE\x10\x02\x12\x14\n\x10HW_SECURE_CRYPTO\x10\x03\x12\x14\n\x10HW_SECURE_DECODE\x10\x04\x12\x11\n\rHW_SECURE_ALL\x10\x05\"\xbd\r\n\x0eLicenseRequest\x12\x44\n\tclient_id\x18\x01 \x01(\x0b\x32\x31.pywidevine_license_protocol.ClientIdentification\x12U\n\ncontent_id\x18\x02 \x01(\x0b\x32\x41.pywidevine_license_protocol.LicenseRequest.ContentIdentification\x12\x45\n\x04type\x18\x03 \x01(\x0e\x32\x37.pywidevine_license_protocol.LicenseRequest.RequestType\x12\x14\n\x0crequest_time\x18\x04 \x01(\x03\x12$\n\x1ckey_control_nonce_deprecated\x18\x05 \x01(\x0c\x12S\n\x10protocol_version\x18\x06 \x01(\x0e\x32,.pywidevine_license_protocol.ProtocolVersion:\x0bVERSION_2_0\x12\x19\n\x11key_control_nonce\x18\x07 \x01(\r\x12W\n\x13\x65ncrypted_client_id\x18\x08 \x01(\x0b\x32:.pywidevine_license_protocol.EncryptedClientIdentification\x1a\x8f\t\n\x15\x43ontentIdentification\x12p\n\x12widevine_pssh_data\x18\x01 \x01(\x0b\x32R.pywidevine_license_protocol.LicenseRequest.ContentIdentification.WidevinePsshDataH\x00\x12\x62\n\x0bwebm_key_id\x18\x02 \x01(\x0b\x32K.pywidevine_license_protocol.LicenseRequest.ContentIdentification.WebmKeyIdH\x00\x12m\n\x10\x65xisting_license\x18\x03 \x01(\x0b\x32Q.pywidevine_license_protocol.LicenseRequest.ContentIdentification.ExistingLicenseH\x00\x12_\n\tinit_data\x18\x04 \x01(\x0b\x32J.pywidevine_license_protocol.LicenseRequest.ContentIdentification.InitDataH\x00\x1ay\n\x10WidevinePsshData\x12\x11\n\tpssh_data\x18\x01 \x03(\x0c\x12>\n\x0clicense_type\x18\x02 \x01(\x0e\x32(.pywidevine_license_protocol.LicenseType\x12\x12\n\nrequest_id\x18\x03 \x01(\x0c\x1ao\n\tWebmKeyId\x12\x0e\n\x06header\x18\x01 \x01(\x0c\x12>\n\x0clicense_type\x18\x02 \x01(\x0e\x32(.pywidevine_license_protocol.LicenseType\x12\x12\n\nrequest_id\x18\x03 \x01(\x0c\x1a\xbe\x01\n\x0f\x45xistingLicense\x12\x46\n\nlicense_id\x18\x01 \x01(\x0b\x32\x32.pywidevine_license_protocol.LicenseIdentification\x12\x1d\n\x15seconds_since_started\x18\x02 \x01(\x03\x12!\n\x19seconds_since_last_played\x18\x03 \x01(\x03\x12!\n\x19session_usage_table_entry\x18\x04 \x01(\x0c\x1a\x8c\x02\n\x08InitData\x12u\n\x0einit_data_type\x18\x01 \x01(\x0e\x32W.pywidevine_license_protocol.LicenseRequest.ContentIdentification.InitData.InitDataType:\x04\x43\x45NC\x12\x11\n\tinit_data\x18\x02 \x01(\x0c\x12>\n\x0clicense_type\x18\x03 \x01(\x0e\x32(.pywidevine_license_protocol.LicenseType\x12\x12\n\nrequest_id\x18\x04 \x01(\x0c\"\"\n\x0cInitDataType\x12\x08\n\x04\x43\x45NC\x10\x01\x12\x08\n\x04WEBM\x10\x02\x42\x14\n\x12\x63ontent_id_variant\"0\n\x0bRequestType\x12\x07\n\x03NEW\x10\x01\x12\x0b\n\x07RENEWAL\x10\x02\x12\x0b\n\x07RELEASE\x10\x03\"\xf3\x01\n\nMetricData\x12\x12\n\nstage_name\x18\x01 \x01(\t\x12\x46\n\x0bmetric_data\x18\x02 \x03(\x0b\x32\x31.pywidevine_license_protocol.MetricData.TypeValue\x1a_\n\tTypeValue\x12@\n\x04type\x18\x01 \x01(\x0e\x32\x32.pywidevine_license_protocol.MetricData.MetricType\x12\x10\n\x05value\x18\x02 \x01(\x03:\x01\x30\"(\n\nMetricType\x12\x0b\n\x07LATENCY\x10\x01\x12\r\n\tTIMESTAMP\x10\x02\"K\n\x0bVersionInfo\x12\x1b\n\x13license_sdk_version\x18\x01 \x01(\t\x12\x1f\n\x17license_service_version\x18\x02 \x01(\t\"\xf6\x05\n\rSignedMessage\x12\x44\n\x04type\x18\x01 \x01(\x0e\x32\x36.pywidevine_license_protocol.SignedMessage.MessageType\x12\x0b\n\x03msg\x18\x02 \x01(\x0c\x12\x11\n\tsignature\x18\x03 \x01(\x0c\x12\x13\n\x0bsession_key\x18\x04 \x01(\x0c\x12\x1a\n\x12remote_attestation\x18\x05 \x01(\x0c\x12<\n\x0bmetric_data\x18\x06 \x03(\x0b\x32\'.pywidevine_license_protocol.MetricData\x12\x46\n\x14service_version_info\x18\x07 \x01(\x0b\x32(.pywidevine_license_protocol.VersionInfo\x12\x64\n\x10session_key_type\x18\x08 \x01(\x0e\x32\x39.pywidevine_license_protocol.SignedMessage.SessionKeyType:\x0fWRAPPED_AES_KEY\x12\x1e\n\x16oemcrypto_core_message\x18\t \x01(\x0c\"\xec\x01\n\x0bMessageType\x12\x13\n\x0fLICENSE_REQUEST\x10\x01\x12\x0b\n\x07LICENSE\x10\x02\x12\x12\n\x0e\x45RROR_RESPONSE\x10\x03\x12\x1f\n\x1bSERVICE_CERTIFICATE_REQUEST\x10\x04\x12\x17\n\x13SERVICE_CERTIFICATE\x10\x05\x12\x0f\n\x0bSUB_LICENSE\x10\x06\x12\x17\n\x13\x43\x41S_LICENSE_REQUEST\x10\x07\x12\x0f\n\x0b\x43\x41S_LICENSE\x10\x08\x12\x1c\n\x18\x45XTERNAL_LICENSE_REQUEST\x10\t\x12\x14\n\x10\x45XTERNAL_LICENSE\x10\n\"S\n\x0eSessionKeyType\x12\r\n\tUNDEFINED\x10\x00\x12\x13\n\x0fWRAPPED_AES_KEY\x10\x01\x12\x1d\n\x19\x45PHERMERAL_ECC_PUBLIC_KEY\x10\x02\"\xc7\x0e\n\x14\x43lientIdentification\x12Q\n\x04type\x18\x01 \x01(\x0e\x32;.pywidevine_license_protocol.ClientIdentification.TokenType:\x06KEYBOX\x12\r\n\x05token\x18\x02 \x01(\x0c\x12P\n\x0b\x63lient_info\x18\x03 \x03(\x0b\x32;.pywidevine_license_protocol.ClientIdentification.NameValue\x12\x1d\n\x15provider_client_token\x18\x04 \x01(\x0c\x12\x17\n\x0flicense_counter\x18\x05 \x01(\r\x12\x61\n\x13\x63lient_capabilities\x18\x06 \x01(\x0b\x32\x44.pywidevine_license_protocol.ClientIdentification.ClientCapabilities\x12\x10\n\x08vmp_data\x18\x07 \x01(\x0c\x12_\n\x12\x64\x65vice_credentials\x18\x08 \x03(\x0b\x32\x43.pywidevine_license_protocol.ClientIdentification.ClientCredentials\x1a(\n\tNameValue\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x1a\xd6\x08\n\x12\x43lientCapabilities\x12\x1b\n\x0c\x63lient_token\x18\x01 \x01(\x08:\x05\x66\x61lse\x12\x1c\n\rsession_token\x18\x02 \x01(\x08:\x05\x66\x61lse\x12+\n\x1cvideo_resolution_constraints\x18\x03 \x01(\x08:\x05\x66\x61lse\x12u\n\x10max_hdcp_version\x18\x04 \x01(\x0e\x32P.pywidevine_license_protocol.ClientIdentification.ClientCapabilities.HdcpVersion:\tHDCP_NONE\x12\x1e\n\x16oem_crypto_api_version\x18\x05 \x01(\r\x12(\n\x19\x61nti_rollback_usage_table\x18\x06 \x01(\x08:\x05\x66\x61lse\x12\x13\n\x0bsrm_version\x18\x07 \x01(\r\x12\x1d\n\x0e\x63\x61n_update_srm\x18\x08 \x01(\x08:\x05\x66\x61lse\x12\x7f\n\x1esupported_certificate_key_type\x18\t \x03(\x0e\x32W.pywidevine_license_protocol.ClientIdentification.ClientCapabilities.CertificateKeyType\x12\x98\x01\n\x1a\x61nalog_output_capabilities\x18\n \x01(\x0e\x32].pywidevine_license_protocol.ClientIdentification.ClientCapabilities.AnalogOutputCapabilities:\x15\x41NALOG_OUTPUT_UNKNOWN\x12(\n\x19\x63\x61n_disable_analog_output\x18\x0b \x01(\x08:\x05\x66\x61lse\x12\x1f\n\x14resource_rating_tier\x18\x0c \x01(\r:\x01\x30\"\x80\x01\n\x0bHdcpVersion\x12\r\n\tHDCP_NONE\x10\x00\x12\x0b\n\x07HDCP_V1\x10\x01\x12\x0b\n\x07HDCP_V2\x10\x02\x12\r\n\tHDCP_V2_1\x10\x03\x12\r\n\tHDCP_V2_2\x10\x04\x12\r\n\tHDCP_V2_3\x10\x05\x12\x1b\n\x16HDCP_NO_DIGITAL_OUTPUT\x10\xff\x01\"i\n\x12\x43\x65rtificateKeyType\x12\x0c\n\x08RSA_2048\x10\x00\x12\x0c\n\x08RSA_3072\x10\x01\x12\x11\n\rECC_SECP256R1\x10\x02\x12\x11\n\rECC_SECP384R1\x10\x03\x12\x11\n\rECC_SECP521R1\x10\x04\"\x8d\x01\n\x18\x41nalogOutputCapabilities\x12\x19\n\x15\x41NALOG_OUTPUT_UNKNOWN\x10\x00\x12\x16\n\x12\x41NALOG_OUTPUT_NONE\x10\x01\x12\x1b\n\x17\x41NALOG_OUTPUT_SUPPORTED\x10\x02\x12!\n\x1d\x41NALOG_OUTPUT_SUPPORTS_CGMS_A\x10\x03\x1au\n\x11\x43lientCredentials\x12Q\n\x04type\x18\x01 \x01(\x0e\x32;.pywidevine_license_protocol.ClientIdentification.TokenType:\x06KEYBOX\x12\r\n\x05token\x18\x02 \x01(\x0c\"s\n\tTokenType\x12\n\n\x06KEYBOX\x10\x00\x12\x1a\n\x16\x44RM_DEVICE_CERTIFICATE\x10\x01\x12\"\n\x1eREMOTE_ATTESTATION_CERTIFICATE\x10\x02\x12\x1a\n\x16OEM_DEVICE_CERTIFICATE\x10\x03\"\xbb\x01\n\x1d\x45ncryptedClientIdentification\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12)\n!service_certificate_serial_number\x18\x02 \x01(\x0c\x12\x1b\n\x13\x65ncrypted_client_id\x18\x03 \x01(\x0c\x12\x1e\n\x16\x65ncrypted_client_id_iv\x18\x04 \x01(\x0c\x12\x1d\n\x15\x65ncrypted_privacy_key\x18\x05 \x01(\x0c\"\xba\x07\n\x0e\x44rmCertificate\x12>\n\x04type\x18\x01 \x01(\x0e\x32\x30.pywidevine_license_protocol.DrmCertificate.Type\x12\x15\n\rserial_number\x18\x02 \x01(\x0c\x12\x1d\n\x15\x63reation_time_seconds\x18\x03 \x01(\r\x12\x1f\n\x17\x65xpiration_time_seconds\x18\x0c \x01(\r\x12\x12\n\npublic_key\x18\x04 \x01(\x0c\x12\x11\n\tsystem_id\x18\x05 \x01(\r\x12\"\n\x16test_device_deprecated\x18\x06 \x01(\x08\x42\x02\x18\x01\x12\x13\n\x0bprovider_id\x18\x07 \x01(\t\x12N\n\rservice_types\x18\x08 \x03(\x0e\x32\x37.pywidevine_license_protocol.DrmCertificate.ServiceType\x12M\n\talgorithm\x18\t \x01(\x0e\x32\x35.pywidevine_license_protocol.DrmCertificate.Algorithm:\x03RSA\x12\x0e\n\x06rot_id\x18\n \x01(\x0c\x12Q\n\x0e\x65ncryption_key\x18\x0b \x01(\x0b\x32\x39.pywidevine_license_protocol.DrmCertificate.EncryptionKey\x1ar\n\rEncryptionKey\x12\x12\n\npublic_key\x18\x01 \x01(\x0c\x12M\n\talgorithm\x18\x02 \x01(\x0e\x32\x35.pywidevine_license_protocol.DrmCertificate.Algorithm:\x03RSA\"L\n\x04Type\x12\x08\n\x04ROOT\x10\x00\x12\x10\n\x0c\x44\x45VICE_MODEL\x10\x01\x12\n\n\x06\x44\x45VICE\x10\x02\x12\x0b\n\x07SERVICE\x10\x03\x12\x0f\n\x0bPROVISIONER\x10\x04\"\x86\x01\n\x0bServiceType\x12\x18\n\x14UNKNOWN_SERVICE_TYPE\x10\x00\x12\x16\n\x12LICENSE_SERVER_SDK\x10\x01\x12\x1c\n\x18LICENSE_SERVER_PROXY_SDK\x10\x02\x12\x14\n\x10PROVISIONING_SDK\x10\x03\x12\x11\n\rCAS_PROXY_SDK\x10\x04\"d\n\tAlgorithm\x12\x15\n\x11UNKNOWN_ALGORITHM\x10\x00\x12\x07\n\x03RSA\x10\x01\x12\x11\n\rECC_SECP256R1\x10\x02\x12\x11\n\rECC_SECP384R1\x10\x03\x12\x11\n\rECC_SECP521R1\x10\x04\"\xce\x01\n\x14SignedDrmCertificate\x12\x17\n\x0f\x64rm_certificate\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x12\x41\n\x06signer\x18\x03 \x01(\x0b\x32\x31.pywidevine_license_protocol.SignedDrmCertificate\x12G\n\x0ehash_algorithm\x18\x04 \x01(\x0e\x32/.pywidevine_license_protocol.HashAlgorithmProto\"\xf6\x05\n\x10WidevinePsshData\x12\x0f\n\x07key_ids\x18\x02 \x03(\x0c\x12\x12\n\ncontent_id\x18\x04 \x01(\x0c\x12\x1b\n\x13\x63rypto_period_index\x18\x07 \x01(\r\x12\x19\n\x11protection_scheme\x18\t \x01(\r\x12\x1d\n\x15\x63rypto_period_seconds\x18\n \x01(\r\x12H\n\x04type\x18\x0b \x01(\x0e\x32\x32.pywidevine_license_protocol.WidevinePsshData.Type:\x06SINGLE\x12\x14\n\x0ckey_sequence\x18\x0c \x01(\r\x12\x11\n\tgroup_ids\x18\r \x03(\x0c\x12P\n\rentitled_keys\x18\x0e \x03(\x0b\x32\x39.pywidevine_license_protocol.WidevinePsshData.EntitledKey\x12\x15\n\rvideo_feature\x18\x0f \x01(\t\x12N\n\talgorithm\x18\x01 \x01(\x0e\x32\x37.pywidevine_license_protocol.WidevinePsshData.AlgorithmB\x02\x18\x01\x12\x14\n\x08provider\x18\x03 \x01(\tB\x02\x18\x01\x12\x16\n\ntrack_type\x18\x05 \x01(\tB\x02\x18\x01\x12\x12\n\x06policy\x18\x06 \x01(\tB\x02\x18\x01\x12\x1b\n\x0fgrouped_license\x18\x08 \x01(\x0c\x42\x02\x18\x01\x1az\n\x0b\x45ntitledKey\x12\x1a\n\x12\x65ntitlement_key_id\x18\x01 \x01(\x0c\x12\x0e\n\x06key_id\x18\x02 \x01(\x0c\x12\x0b\n\x03key\x18\x03 \x01(\x0c\x12\n\n\x02iv\x18\x04 \x01(\x0c\x12&\n\x1a\x65ntitlement_key_size_bytes\x18\x05 \x01(\r:\x02\x33\x32\"5\n\x04Type\x12\n\n\x06SINGLE\x10\x00\x12\x0f\n\x0b\x45NTITLEMENT\x10\x01\x12\x10\n\x0c\x45NTITLED_KEY\x10\x02\"(\n\tAlgorithm\x12\x0f\n\x0bUNENCRYPTED\x10\x00\x12\n\n\x06\x41\x45SCTR\x10\x01\"\xb0\x06\n\x07VmpData\x12\x14\n\x0c\x63\x65rtificates\x18\x01 \x03(\x0c\x12Q\n\x12signed_binary_info\x18\x02 \x03(\x0b\x32\x35.pywidevine_license_protocol.VmpData.SignedBinaryInfo\x12M\n\x0e\x63\x64m_host_files\x18\x03 \x03(\x0b\x32\x35.pywidevine_license_protocol.VmpData.SignedBinaryInfo\x12S\n\x14\x63\x64m_call_chain_files\x18\x04 \x03(\x0b\x32\x35.pywidevine_license_protocol.VmpData.SignedBinaryInfo\x12S\n\x14\x63urrent_process_file\x18\x05 \x01(\x0b\x32\x35.pywidevine_license_protocol.VmpData.SignedBinaryInfo\x12\x19\n\x11host_file_indexes\x18\x06 \x03(\r\x12\x1f\n\x17\x63\x61ll_chain_file_indexes\x18\x07 \x03(\r\x12\"\n\x1a\x63urrent_process_file_index\x18\x08 \x03(\r\x1a\xe2\x02\n\x10SignedBinaryInfo\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x19\n\x11\x63\x65rtificate_index\x18\x02 \x01(\r\x12\x13\n\x0b\x62inary_hash\x18\x03 \x01(\x0c\x12\r\n\x05\x66lags\x18\x04 \x01(\r\x12\x11\n\tsignature\x18\x05 \x01(\x0c\x12`\n\x0ehash_algorithm\x18\x06 \x01(\x0e\x32H.pywidevine_license_protocol.VmpData.SignedBinaryInfo.HashAlgorithmProto\"\x86\x01\n\x12HashAlgorithmProto\x12\x1e\n\x1aHASH_ALGORITHM_UNSPECIFIED\x10\x00\x12\x18\n\x14HASH_ALGORITHM_SHA_1\x10\x01\x12\x1a\n\x16HASH_ALGORITHM_SHA_256\x10\x02\x12\x1a\n\x16HASH_ALGORITHM_SHA_384\x10\x03*8\n\x0bLicenseType\x12\r\n\tSTREAMING\x10\x01\x12\x0b\n\x07OFFLINE\x10\x02\x12\r\n\tAUTOMATIC\x10\x03*\xd9\x01\n\x1aPlatformVerificationStatus\x12\x17\n\x13PLATFORM_UNVERIFIED\x10\x00\x12\x15\n\x11PLATFORM_TAMPERED\x10\x01\x12\x1e\n\x1aPLATFORM_SOFTWARE_VERIFIED\x10\x02\x12\x1e\n\x1aPLATFORM_HARDWARE_VERIFIED\x10\x03\x12\x1c\n\x18PLATFORM_NO_VERIFICATION\x10\x04\x12-\n)PLATFORM_SECURE_STORAGE_SOFTWARE_VERIFIED\x10\x05*D\n\x0fProtocolVersion\x12\x0f\n\x0bVERSION_2_0\x10\x14\x12\x0f\n\x0bVERSION_2_1\x10\x15\x12\x0f\n\x0bVERSION_2_2\x10\x16*\x86\x01\n\x12HashAlgorithmProto\x12\x1e\n\x1aHASH_ALGORITHM_UNSPECIFIED\x10\x00\x12\x18\n\x14HASH_ALGORITHM_SHA_1\x10\x01\x12\x1a\n\x16HASH_ALGORITHM_SHA_256\x10\x02\x12\x1a\n\x16HASH_ALGORITHM_SHA_384\x10\x03\x42\x02H\x03')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'license_protocol_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\003'
  _globals['_DRMCERTIFICATE'].fields_by_name['test_device_deprecated']._loaded_options = None
  _globals['_DRMCERTIFICATE'].fields_by_name['test_device_deprecated']._serialized_options = b'\030\001'
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['algorithm']._loaded_options = None
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['algorithm']._serialized_options = b'\030\001'
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['provider']._loaded_options = None
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['provider']._serialized_options = b'\030\001'
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['track_type']._loaded_options = None
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['track_type']._serialized_options = b'\030\001'
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['policy']._loaded_options = None
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['policy']._serialized_options = b'\030\001'
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['grouped_license']._loaded_options = None
  _globals['_WIDEVINEPSSHDATA'].fields_by_name['grouped_license']._serialized_options = b'\030\001'
  _globals['_LICENSETYPE']._serialized_start=11049
  _globals['_LICENSETYPE']._serialized_end=11105
  _globals['_PLATFORMVERIFICATIONSTATUS']._serialized_start=11108
  _globals['_PLATFORMVERIFICATIONSTATUS']._serialized_end=11325
  _globals['_PROTOCOLVERSION']._serialized_start=11327
  _globals['_PROTOCOLVERSION']._serialized_end=11395
  _globals['_HASHALGORITHMPROTO']._serialized_start=10913
  _globals['_HASHALGORITHMPROTO']._serialized_end=11047
  _globals['_LICENSEIDENTIFICATION']._serialized_start=56
  _globals['_LICENSEIDENTIFICATION']._serialized_end=245
  _globals['_LICENSE']._serialized_start=248
  _globals['_LICENSE']._serialized_end=3433
  _globals['_LICENSE_POLICY']._serialized_start=764
  _globals['_LICENSE_POLICY']._serialized_end=1322
  _globals['_LICENSE_KEYCONTAINER']._serialized_start=1325
  _globals['_LICENSE_KEYCONTAINER']._serialized_end=3433
  _globals['_LICENSE_KEYCONTAINER_KEYCONTROL']._serialized_start=2128
  _globals['_LICENSE_KEYCONTAINER_KEYCONTROL']._serialized_end=2179
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION']._serialized_start=2182
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION']._serialized_end=2850
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION_HDCP']._serialized_start=2604
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION_HDCP']._serialized_end=2725
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION_CGMS']._serialized_start=2727
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION_CGMS']._serialized_end=2794
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION_HDCPSRMRULE']._serialized_start=2796
  _globals['_LICENSE_KEYCONTAINER_OUTPUTPROTECTION_HDCPSRMRULE']._serialized_end=2850
  _globals['_LICENSE_KEYCONTAINER_VIDEORESOLUTIONCONSTRAINT']._serialized_start=2853
  _globals['_LICENSE_KEYCONTAINER_VIDEORESOLUTIONCONSTRAINT']._serialized_end=3039
  _globals['_LICENSE_KEYCONTAINER_OPERATORSESSIONKEYPERMISSIONS']._serialized_start=3042
  _globals['_LICENSE_KEYCONTAINER_OPERATORSESSIONKEYPERMISSIONS']._serialized_end=3199
  _globals['_LICENSE_KEYCONTAINER_KEYTYPE']._serialized_start=3201
  _globals['_LICENSE_KEYCONTAINER_KEYTYPE']._serialized_end=3309
  _globals['_LICENSE_KEYCONTAINER_SECURITYLEVEL']._serialized_start=3311
  _globals['_LICENSE_KEYCONTAINER_SECURITYLEVEL']._serialized_end=3433
  _globals['_LICENSEREQUEST']._serialized_start=3436
  _globals['_LICENSEREQUEST']._serialized_end=5161
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION']._serialized_start=3944
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION']._serialized_end=5111
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_WIDEVINEPSSHDATA']._serialized_start=4391
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_WIDEVINEPSSHDATA']._serialized_end=4512
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_WEBMKEYID']._serialized_start=4514
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_WEBMKEYID']._serialized_end=4625
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_EXISTINGLICENSE']._serialized_start=4628
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_EXISTINGLICENSE']._serialized_end=4818
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_INITDATA']._serialized_start=4821
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_INITDATA']._serialized_end=5089
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_INITDATA_INITDATATYPE']._serialized_start=5055
  _globals['_LICENSEREQUEST_CONTENTIDENTIFICATION_INITDATA_INITDATATYPE']._serialized_end=5089
  _globals['_LICENSEREQUEST_REQUESTTYPE']._serialized_start=5113
  _globals['_LICENSEREQUEST_REQUESTTYPE']._serialized_end=5161
  _globals['_METRICDATA']._serialized_start=5164
  _globals['_METRICDATA']._serialized_end=5407
  _globals['_METRICDATA_TYPEVALUE']._serialized_start=5270
  _globals['_METRICDATA_TYPEVALUE']._serialized_end=5365
  _globals['_METRICDATA_METRICTYPE']._serialized_start=5367
  _globals['_METRICDATA_METRICTYPE']._serialized_end=5407
  _globals['_VERSIONINFO']._serialized_start=5409
  _globals['_VERSIONINFO']._serialized_end=5484
  _globals['_SIGNEDMESSAGE']._serialized_start=5487
  _globals['_SIGNEDMESSAGE']._serialized_end=6245
  _globals['_SIGNEDMESSAGE_MESSAGETYPE']._serialized_start=5924
  _globals['_SIGNEDMESSAGE_MESSAGETYPE']._serialized_end=6160
  _globals['_SIGNEDMESSAGE_SESSIONKEYTYPE']._serialized_start=6162
  _globals['_SIGNEDMESSAGE_SESSIONKEYTYPE']._serialized_end=6245
  _globals['_CLIENTIDENTIFICATION']._serialized_start=6248
  _globals['_CLIENTIDENTIFICATION']._serialized_end=8111
  _globals['_CLIENTIDENTIFICATION_NAMEVALUE']._serialized_start=6722
  _globals['_CLIENTIDENTIFICATION_NAMEVALUE']._serialized_end=6762
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES']._serialized_start=6765
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES']._serialized_end=7875
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES_HDCPVERSION']._serialized_start=7496
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES_HDCPVERSION']._serialized_end=7624
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES_CERTIFICATEKEYTYPE']._serialized_start=7626
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES_CERTIFICATEKEYTYPE']._serialized_end=7731
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES_ANALOGOUTPUTCAPABILITIES']._serialized_start=7734
  _globals['_CLIENTIDENTIFICATION_CLIENTCAPABILITIES_ANALOGOUTPUTCAPABILITIES']._serialized_end=7875
  _globals['_CLIENTIDENTIFICATION_CLIENTCREDENTIALS']._serialized_start=7877
  _globals['_CLIENTIDENTIFICATION_CLIENTCREDENTIALS']._serialized_end=7994
  _globals['_CLIENTIDENTIFICATION_TOKENTYPE']._serialized_start=7996
  _globals['_CLIENTIDENTIFICATION_TOKENTYPE']._serialized_end=8111
  _globals['_ENCRYPTEDCLIENTIDENTIFICATION']._serialized_start=8114
  _globals['_ENCRYPTEDCLIENTIDENTIFICATION']._serialized_end=8301
  _globals['_DRMCERTIFICATE']._serialized_start=8304
  _globals['_DRMCERTIFICATE']._serialized_end=9258
  _globals['_DRMCERTIFICATE_ENCRYPTIONKEY']._serialized_start=8827
  _globals['_DRMCERTIFICATE_ENCRYPTIONKEY']._serialized_end=8941
  _globals['_DRMCERTIFICATE_TYPE']._serialized_start=8943
  _globals['_DRMCERTIFICATE_TYPE']._serialized_end=9019
  _globals['_DRMCERTIFICATE_SERVICETYPE']._serialized_start=9022
  _globals['_DRMCERTIFICATE_SERVICETYPE']._serialized_end=9156
  _globals['_DRMCERTIFICATE_ALGORITHM']._serialized_start=9158
  _globals['_DRMCERTIFICATE_ALGORITHM']._serialized_end=9258
  _globals['_SIGNEDDRMCERTIFICATE']._serialized_start=9261
  _globals['_SIGNEDDRMCERTIFICATE']._serialized_end=9467
  _globals['_WIDEVINEPSSHDATA']._serialized_start=9470
  _globals['_WIDEVINEPSSHDATA']._serialized_end=10228
  _globals['_WIDEVINEPSSHDATA_ENTITLEDKEY']._serialized_start=10009
  _globals['_WIDEVINEPSSHDATA_ENTITLEDKEY']._serialized_end=10131
  _globals['_WIDEVINEPSSHDATA_TYPE']._serialized_start=10133
  _globals['_WIDEVINEPSSHDATA_TYPE']._serialized_end=10186
  _globals['_WIDEVINEPSSHDATA_ALGORITHM']._serialized_start=10188
  _globals['_WIDEVINEPSSHDATA_ALGORITHM']._serialized_end=10228
  _globals['_VMPDATA']._serialized_start=10231
  _globals['_VMPDATA']._serialized_end=11047
  _globals['_VMPDATA_SIGNEDBINARYINFO']._serialized_start=10693
  _globals['_VMPDATA_SIGNEDBINARYINFO']._serialized_end=11047
  _globals['_VMPDATA_SIGNEDBINARYINFO_HASHALGORITHMPROTO']._serialized_start=10913
  _globals['_VMPDATA_SIGNEDBINARYINFO_HASHALGORITHMPROTO']._serialized_end=11047
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import base64
import binascii
import string
from io import BytesIO
from typing import Optional, Union
from uuid import UUID
from xml.etree.ElementTree import XML

import construct
from construct import Container
from google.protobuf.message import DecodeError
from pymp4.parser import Box

from pywidevine.license_protocol_pb2 import WidevinePsshData


class PSSH:
    """
    MP4 PSSH Box-related utilities.
    Allows you to load, create, and modify various kinds of DRM system headers.
    """

    class SystemId:
        Widevine = UUID(hex="edef8ba979d64acea3c827dcd51d21ed")
        PlayReady = UUID(hex="9a04f07998404286ab92e65be0885f95")

    def __init__(self, data: Union[Container, str, bytes], strict: bool = False):
        """
        Load a PSSH box, WidevineCencHeader, or PlayReadyHeader.

        When loading a WidevineCencHeader or PlayReadyHeader, a new v0 PSSH box will be
        created and the header will be parsed and stored in the init_data field. However,
        PlayReadyHeaders (and PlayReadyObjects) are not yet currently parsed and are
        stored as bytes.

        [Strict mode (strict=True)]

        Supports the following forms of input data in either Base64 or Bytes form:
        - Full PSSH mp4 boxes (as defined by pymp4 Box).
        - Full Widevine Cenc Headers (as defined by WidevinePsshData proto).
        - Full PlayReady Objects and Headers (as defined by Microsoft Docs).

        [Lenient mode (strict=False, default)]

        If the data is not supported in Strict mode, and is assumed not to be corrupt or
        parsed incorrectly, the License Server likely accepts a custom init_data value
        during a License Request call. This is uncommon behavior but not out of realm of
        possibilities. For example, Netflix does this with it's MSL WidevineExchange
        scheme.

        Lenient mode will craft a new v0 PSSH box with the init_data field set to
        the provided data as-is. The data will first be base64 decoded. This behavior
        may not work in your scenario and if that's the case please manually craft
        your own PSSH box with the init_data field to be used in License Requests.

        Raises:
            ValueError: If the data is empty.
            TypeError: If the data is an unexpected type.
            binascii.Error: If the data could not be decoded as Base64 if provided as a
                string.
            DecodeError: If the data could not be parsed as a PSSH mp4 box nor a Widevine
                Cenc Header and strict mode is enabled.
        """
        if not data:
            raise ValueError("Data must not be empty.")

        if isinstance(data, Container):
            box = data
        else:
            if isinstance(data, str):
                try:
                    data = base64.b64decode(data)
                except (binascii.Error, binascii.Incomplete) as e:
                    raise binascii.Error(f"Could not decode data as Base64, {e}")

            if not isinstance(data, bytes):
                raise TypeError(f"Expected data to be a {Container}, bytes, or base64, not {data!r}")

            try:
                box = Box.parse(data)
            except (IOError, construct.ConstructError):  # not a box
                try:
                    widevine_pssh_data = WidevinePsshData()
                    widevine_pssh_data.ParseFromString(data)
                    data_serialized = widevine_pssh_data.SerializeToString()
                    if data_serialized != data:  # not actually a WidevinePsshData
                        raise DecodeError()
                    box = Box.parse(Box.build(dict(
                        type=b"pssh",
                        version=0,
                        flags=0,
                        system_ID=PSSH.SystemId.Widevine,
                        init_data=data_serialized
                    )))
                except DecodeError:  # not a widevine cenc header
                    if "</WRMHEADER>".encode("utf-16-le") in data:
                        # TODO: Actually parse `data` as a PlayReadyHeader object and store that instead
                        box = Box.parse(Box.build(dict(
                            type=b"pssh",
                            version=0,
                            flags=0,
                            system_ID=PSSH.SystemId.PlayReady,
                            init_data=data
                        )))
                    elif strict:
                        raise DecodeError(f"Could not parse data as a {Container} nor a {WidevinePsshData}.")
                    else:
                        # Data is not a WidevineCencHeader nor a PlayReadyHeader.
                        # The license server likely has something custom to parse it.
                        # See doc-string about Lenient mode for more information.
                        box = Box.parse(Box.build(dict(
                            type=b"pssh",
                            version=0,
                            flags=0,
                            system_ID=PSSH.SystemId.Widevine,
                            init_data=data
                        )))

        self.version = box.version
        self.flags = box.flags
        self.system_id = box.system_ID
        self.__key_ids = box.key_IDs
        self.init_data = box.init_data

    def __repr__(self) -> str:
        return f"PSSH<{self.system_id}>(v{self.version}; {self.flags}, {self.key_ids}, {self.init_data})"

    def __str__(self) -> str:
        return self.dumps()

    @classmethod
    def new(
        cls,
        system_id: UUID,
        key_ids: Optional[list[Union[UUID, str, bytes]]] = None,
        init_data: Optional[Union[WidevinePsshData, str, bytes]] = None,
        version: int = 0,
        flags: int = 0
    ) -> PSSH:
        """Craft a new version 0 or 1 PSSH Box."""
        if not system_id:
            raise ValueError("A System ID must be specified.")
        if not isinstance(system_id, UUID):
            raise TypeError(f"Expected system_id to be a UUID, not {system_id!r}")

        if key_ids is not None and not isinstance(key_ids, list):
            raise TypeError(f"Expected key_ids to be a list not {key_ids!r}")

        if init_data is not None and not isinstance(init_data, (WidevinePsshData, str, bytes)):
            raise TypeError(f"Expected init_data to be a {WidevinePsshData}, base64, or bytes, not {init_data!r}")

        if not isinstance(version, int):
            raise TypeError(f"Expected version to be an int not {version!r}")
        if version not in (0, 1):
            raise ValueError(f"Invalid version, must be either 0 or 1, not {version}.")

        if not isinstance(flags, int):
            raise TypeError(f"Expected flags to be an int not {flags!r}")
        if flags < 0:
            raise ValueError("Invalid flags, cannot be less than 0.")

        if version == 0 and key_ids is not None and init_data is not None:
            # v0 boxes use only init_data in the pssh field, but we can use the key_ids within the init_data
            raise ValueError("Version 0 PSSH boxes must use only init_data, not init_data and key_ids.")
        elif version == 1:
            # TODO: I cannot tell if they need either init_data or key_ids exclusively, or both is fine
            #       So for now I will just make sure at least one is supplied
            if init_data is None and key_ids is None:
                raise ValueError("Version 1 PSSH boxes must use either init_data or key_ids but neither were provided")

        if init_data is not None:
            if isinstance(init_data, WidevinePsshData):
                init_data = init_data.SerializeToString()
            elif isinstance(init_data, str):
                if all(c in string.hexdigits for c in init_data):
                    init_data = bytes.fromhex(init_data)
                else:
                    init_data = base64.b64decode(init_data)
            elif not isinstance(init_data, bytes):
                raise TypeError(
                    f"Expecting init_data to be {WidevinePsshData}, hex, base64, or bytes, not {init_data!r}"
                )

        pssh = cls(Box.parse(Box.build(dict(
            type=b"pssh",
            version=version,
            flags=flags,
            system_ID=system_id,
            init_data=[init_data, b""][init_data is None]
            # key_IDs should not be set yet
        ))))

        if key_ids:
            # We must reinforce the version because pymp4 forces v0 if key_IDs is not set.
            # The set_key_ids() func will set it efficiently in both init_data and the box where needed.
            # The version must be reinforced ONLY if we have key_id data or there's a possibility of making
            # a v1 PSSH box, that did not have key_IDs set in the PSSH box.
            pssh.version = version
            pssh.set_key_ids(key_ids)

        return pssh

    @property
    def key_ids(self) -> list[UUID]:
        """
        Get all Key IDs from within the Box or Init Data, wherever possible.

        Supports:
        - Version 1 PSSH Boxes
        - WidevineCencHeaders
        - PlayReadyHeaders (4.0.0.0->4.3.0.0)
        """
        if self.version == 1 and self.__key_ids:
            return self.__key_ids

        if self.system_id == PSSH.SystemId.Widevine:
            # TODO: What if its not a Widevine Cenc Header but the System ID is set as Widevine?
            cenc_header = WidevinePsshData()
            cenc_header.ParseFromString(self.init_data)
            return [
                # the key_ids value may or may not be hex underlying
                (
                    UUID(bytes=key_id) if len(key_id) == 16 else  # normal
                    UUID(hex=key_id.decode()) if len(key_id) == 32 else  # stored as hex
                    UUID(int=int.from_bytes(key_id, "big"))  # assuming as number
                )
                for key_id in cenc_header.key_ids
            ]

        if self.system_id == PSSH.SystemId.PlayReady:
            # Assuming init data is a PRO (PlayReadyObject)
            # https://learn.microsoft.com/en-us/playready/specifications/playready-header-specification
            pro_data = BytesIO(self.init_data)
            pro_length = int.from_bytes(pro_data.read(4), "little")
            if pro_length != len(self.init_data):
                raise ValueError("The PlayReadyObject seems to be corrupt (too big or small, or missing data).")
            pro_record_count = int.from_bytes(pro_data.read(2), "little")

            for _ in range(pro_record_count):
                prr_type = int.from_bytes(pro_data.read(2), "little")
                prr_length = int.from_bytes(pro_data.read(2), "little")
                prr_value = pro_data.read(prr_length)
                if prr_type != 0x01:
                    # No PlayReady Header, skip and hope for something else
                    # TODO: Add support for Embedded License Stores (0x03)
                    continue

                wrm_ns = {"wrm": "http://schemas.microsoft.com/DRM/2007/03/PlayReadyHeader"}
                prr_header = XML(prr_value.decode("utf-16-le"))
                prr_header_version = prr_header.get("version")
                if prr_header_version == "4.0.0.0":
                    key_ids = [
                        x.text
                        for x in prr_header.findall("./wrm:DATA/wrm:KID", wrm_ns)
                        if x.text
                    ]
                elif prr_header_version == "4.1.0.0":
                    key_ids = [
                        x.attrib["VALUE"]
                        for x in prr_header.findall("./wrm:DATA/wrm:PROTECTINFO/wrm:KID", wrm_ns)
                    ]
                elif prr_header_version in ("4.2.0.0", "4.3.0.0"):
                    # TODO: Retain the Encryption Scheme information in v4.3.0.0
                    #       This is because some Key IDs can be AES-CTR while some are AES-CBC.
                    #       Conversion to WidevineCencHeader could use this information.
                    key_ids = [
                        x.attrib["VALUE"]
                        for x in prr_header.findall("./wrm:DATA/wrm:PROTECTINFO/wrm:KIDS/wrm:KID", wrm_ns)
                    ]
                else:
                    raise ValueError(f"Unsupported PlayReadyHeader version {prr_header_version}")

                return [
                    UUID(bytes=base64.b64decode(key_id))
                    for key_id in key_ids
                ]

            raise ValueError("Unsupported PlayReadyObject, no PlayReadyHeader within the object.")

        raise ValueError(f"This PSSH is not supported by key_ids() property, {self.dumps()}")

    def dump(self) -> bytes:
        """Export the PSSH object as a full PSSH box in bytes form."""
        return Box.build(dict(
            type=b"pssh",
            version=self.version,
            flags=self.flags,
            system_ID=self.system_id,
            key_IDs=self.key_ids if self.version == 1 and self.key_ids else None,
            init_data=self.init_data
        ))

    def dumps(self) -> str:
        """Export the PSSH object as a full PSSH box in base64 form."""
        return base64.b64encode(self.dump()).decode()

    def to_widevine(self) -> None:
        """
        Convert PlayReady PSSH data to Widevine PSSH data.

        There's only a limited amount of information within a PlayReady PSSH header that
        can be used in a Widevine PSSH Header. The converted data may or may not result
        in an accepted PSSH. It depends on what the License Server is expecting.
        """
        if self.system_id == PSSH.SystemId.Widevine:
            raise ValueError("This is already a Widevine PSSH")

        widevine_pssh_data = WidevinePsshData(
            key_ids=[x.bytes for x in self.key_ids],
            algorithm="AESCTR"
        )

        if self.version == 1:
            # ensure both cenc header and box has same Key IDs
            # v1 uses both this and within init data for basically no reason
            self.__key_ids = self.key_ids

        self.init_data = widevine_pssh_data.SerializeToString()
        self.system_id = PSSH.SystemId.Widevine

    def to_playready(
        self,
        la_url: Optional[str] = None,
        lui_url: Optional[str] = None,
        ds_id: Optional[bytes] = None,
        decryptor_setup: Optional[str] = None,
        custom_data: Optional[str] = None
    ) -> None:
        """
        Convert Widevine PSSH data to PlayReady v4.3.0.0 PSSH data.

        Note that it is impossible to create the CHECKSUM values for AES-CTR Key IDs
        as you must encrypt the Key ID with the Content Encryption Key using AES-ECB.
        This may cause software incompatibilities.

        Parameters:
            la_url: Contains the URL for the license acquisition Web service.
                Only absolute URLs are allowed.
            lui_url: Contains the URL for the license acquisition Web service.
                Only absolute URLs are allowed.
            ds_id: Service ID for the domain service.
            decryptor_setup: This tag may only contain the value "ONDEMAND". It
                indicates to an application that it should not expect the full
                license chain for the content to be available for acquisition, or
                already present on the client machine, prior to setting up the
                media graph. If this tag is not set then it indicates that an
                application can enforce the license to be acquired, or already
                present on the client machine, prior to setting up the media graph.
            custom_data: The content author can add custom XML inside this
                element. Microsoft code does not act on any data contained inside
                this element. The Syntax of this params XML is not validated.
        """
        if self.system_id == PSSH.SystemId.PlayReady:
            raise ValueError("This is already a PlayReady PSSH")

        key_ids_xml = ""
        for key_id in self.key_ids:
            # Note that it's impossible to create the CHECKSUM value without the Key for the KID
            key_ids_xml += f"""
            <KID ALGID="AESCTR" VALUE="{base64.b64encode(key_id.bytes).decode()}"></KID>
            """

        prr_value = f"""
        <WRMHEADER xmlns="http://schemas.microsoft.com/DRM/2007/03/PlayReadyHeader" version="4.3.0.0">
            <DATA>
                <PROTECTINFO>
                    <KIDS>{key_ids_xml}</KIDS>
                </PROTECTINFO>
                {'<LA_URL>%s</LA_URL>' % la_url if la_url else ''}
                {'<LUI_URL>%s</LUI_URL>' % lui_url if lui_url else ''}
                {'<DS_ID>%s</DS_ID>' % base64.b64encode(ds_id).decode() if ds_id else ''}
                {'<DECRYPTORSETUP>%s</DECRYPTORSETUP>' % decryptor_setup if decryptor_setup else ''}
                {'<CUSTOMATTRIBUTES xmlns="">%s</CUSTOMATTRIBUTES>' % custom_data if custom_data else ''}
            </DATA>
        </WRMHEADER>
        """.encode("utf-16-le")

        prr_length = len(prr_value).to_bytes(2, "little")
        prr_type = (1).to_bytes(2, "little")  # Has PlayReadyHeader
        pro_record_count = (1).to_bytes(2, "little")
        pro = pro_record_count + prr_type + prr_length + prr_value
        pro = (len(pro) + 4).to_bytes(4, "little") + pro

        self.init_data = pro
        self.system_id = PSSH.SystemId.PlayReady

    def set_key_ids(self, key_ids: list[Union[UUID, str, bytes]]) -> None:
        """Overwrite all Key IDs with the specified Key IDs."""
        if self.system_id != PSSH.SystemId.Widevine:
            # TODO: Add support for setting the Key IDs in a PlayReady Header
            raise ValueError(f"Only Widevine PSSH Boxes are supported, not {self.system_id}.")

        key_id_uuids = self.parse_key_ids(key_ids)

        if self.version == 1 or self.__key_ids:
            # only use v1 box key_ids if version is 1, or it's already being used
            # this is in case the service stupidly expects it for version 0
            self.__key_ids = key_id_uuids

        cenc_header = WidevinePsshData()
        cenc_header.ParseFromString(self.init_data)

        cenc_header.key_ids[:] = [
            key_id.bytes
            for key_id in key_id_uuids
        ]

        self.init_data = cenc_header.SerializeToString()

    @staticmethod
    def parse_key_ids(key_ids: list[Union[UUID, str, bytes]]) -> list[UUID]:
        """
        Parse a list of Key IDs in hex, base64, or bytes to UUIDs.

        Raises TypeError if `key_ids` is not a list, or the list contains one
        or more items that are not a UUID, str, or bytes object.
        """
        if not isinstance(key_ids, list):
            raise TypeError(f"Expected key_ids to be a list, not {key_ids!r}")

        if not all(isinstance(x, (UUID, str, bytes)) for x in key_ids):
            raise TypeError("Some items of key_ids are not a UUID, str, or bytes. Unsure how to continue...")

        uuids = [
            UUID(bytes=key_id_b)
            for key_id in key_ids
            for key_id_b in [
                key_id.bytes if isinstance(key_id, UUID) else
                (
                    bytes.fromhex(key_id) if all(c in string.hexdigits for c in key_id) else
                    base64.b64decode(key_id)
                ) if isinstance(key_id, str) else
                key_id
            ]
        ]

        return uuids


__all__ = ("PSSH",)



from __future__ import annotations

import base64
import binascii
from typing import Optional, Union

import requests
from Crypto.Hash import SHA1
from Crypto.PublicKey import RSA
from Crypto.Signature import pss
from google.protobuf.message import DecodeError

from pywidevine.cdm import Cdm
from pywidevine.device import Device, DeviceTypes
from pywidevine.exceptions import (InvalidInitData, InvalidLicenseMessage, InvalidLicenseType,
                                   SignatureMismatch)
from pywidevine.key import Key
from pywidevine.license_protocol_pb2 import (ClientIdentification, License, LicenseType, SignedDrmCertificate,
                                             SignedMessage)
from pywidevine.pssh import PSSH


class RemoteCdm(Cdm):
    """Remote Accessible CDM using pywidevine's serve schema."""

    def __init__(
        self,
        device_type: Union[DeviceTypes, str],
        system_id: int,
        security_level: int,
        host: str,
        secret: str,
        device_name: str
    ):
        """Initialize a Widevine Content Decryption Module (CDM)."""
        if not device_type:
            raise ValueError("Device Type must be provided")
        if isinstance(device_type, str):
            device_type = DeviceTypes[device_type]
        if not isinstance(device_type, DeviceTypes):
            raise TypeError(f"Expected device_type to be a {DeviceTypes!r} not {device_type!r}")

        if not system_id:
            raise ValueError("System ID must be provided")
        if not isinstance(system_id, int):
            raise TypeError(f"Expected system_id to be a {int} not {system_id!r}")

        if not security_level:
            raise ValueError("Security Level must be provided")
        if not isinstance(security_level, int):
            raise TypeError(f"Expected security_level to be a {int} not {security_level!r}")

        if not host:
            raise ValueError("API Host must be provided")
        if not isinstance(host, str):
            raise TypeError(f"Expected host to be a {str} not {host!r}")

        if not secret:
            raise ValueError("API Secret must be provided")
        if not isinstance(secret, str):
            raise TypeError(f"Expected secret to be a {str} not {secret!r}")

        if not device_name:
            raise ValueError("API Device name must be provided")
        if not isinstance(device_name, str):
            raise TypeError(f"Expected device_name to be a {str} not {device_name!r}")

        self.device_type = device_type
        self.system_id = system_id
        self.security_level = security_level
        self.host = host
        self.device_name = device_name

        # spoof client_id and rsa_key just so we can construct via super call
        super().__init__(device_type, system_id, security_level, ClientIdentification(), RSA.generate(2048))

        self.__session = requests.Session()
        self.__session.headers.update({
            "X-Secret-Key": secret
        })

        r = requests.head(self.host)
        if r.status_code != 200:
            raise ValueError(f"Could not test Remote API version [{r.status_code}]")

    @classmethod
    def from_device(cls, device: Device) -> RemoteCdm:
        raise NotImplementedError("You cannot load a RemoteCdm from a local Device file.")

    def open(self) -> bytes:
        r = self.__session.get(
            url=f"{self.host}/{self.device_name}/open"
        ).json()
        if r['status'] != 200:
            raise ValueError(f"Cannot Open CDM Session, {r['message']} [{r['status']}]")
        r = r["data"]

        return bytes.fromhex(r["session_id"])

    def close(self, session_id: bytes) -> None:
        r = self.__session.get(
            url=f"{self.host}/{self.device_name}/close/{session_id.hex()}"
        ).json()
        if r["status"] != 200:
            raise ValueError(f"Cannot Close CDM Session, {r['message']} [{r['status']}]")

    def set_service_certificate(self, session_id: bytes, certificate: Optional[Union[bytes, str]]) -> str:
        if certificate is None:
            certificate_b64 = None
        elif isinstance(certificate, str):
            certificate_b64 = certificate  # assuming base64
        elif isinstance(certificate, bytes):
            certificate_b64 = base64.b64encode(certificate).decode()
        else:
            raise DecodeError(f"Expecting Certificate to be base64 or bytes, not {certificate!r}")

        r = self.__session.post(
            url=f"{self.host}/{self.device_name}/set_service_certificate",
            json={
                "session_id": session_id.hex(),
                "certificate": certificate_b64
            }
        ).json()
        if r["status"] != 200:
            raise ValueError(f"Cannot Set CDMs Service Certificate, {r['message']} [{r['status']}]")
        r = r["data"]

        return r["provider_id"]

    def get_service_certificate(self, session_id: bytes) -> Optional[SignedDrmCertificate]:
        r = self.__session.post(
            url=f"{self.host}/{self.device_name}/get_service_certificate",
            json={
                "session_id": session_id.hex()
            }
        ).json()
        if r["status"] != 200:
            raise ValueError(f"Cannot Get CDMs Service Certificate, {r['message']} [{r['status']}]")
        r = r["data"]

        service_certificate = r["service_certificate"]
        if not service_certificate:
            return None

        service_certificate = base64.b64decode(service_certificate)
        signed_drm_certificate = SignedDrmCertificate()

        try:
            signed_drm_certificate.ParseFromString(service_certificate)
            if signed_drm_certificate.SerializeToString() != service_certificate:
                raise DecodeError("partial parse")
        except DecodeError as e:
            # could be a direct unsigned DrmCertificate, but reject those anyway
            raise DecodeError(f"Could not parse certificate as a SignedDrmCertificate, {e}")

        try:
            pss. \
                new(RSA.import_key(self.root_cert.public_key)). \
                verify(
                    msg_hash=SHA1.new(signed_drm_certificate.drm_certificate),
                    signature=signed_drm_certificate.signature
                )
        except (ValueError, TypeError):
            raise SignatureMismatch("Signature Mismatch on SignedDrmCertificate, rejecting certificate")

        return signed_drm_certificate

    def get_license_challenge(
        self,
        session_id: bytes,
        pssh: PSSH,
        license_type: str = "STREAMING",
        privacy_mode: bool = True
    ) -> bytes:
        if not pssh:
            raise InvalidInitData("A pssh must be provided.")
        if not isinstance(pssh, PSSH):
            raise InvalidInitData(f"Expected pssh to be a {PSSH}, not {pssh!r}")

        if not isinstance(license_type, str):
            raise InvalidLicenseType(f"Expected license_type to be a {str}, not {license_type!r}")
        if license_type not in LicenseType.keys():
            raise InvalidLicenseType(
                f"Invalid license_type value of '{license_type}'. "
                f"Available values: {LicenseType.keys()}"
            )

        r = self.__session.post(
            url=f"{self.host}/{self.device_name}/get_license_challenge/{license_type}",
            json={
                "session_id": session_id.hex(),
                "init_data": pssh.dumps(),
                "privacy_mode": privacy_mode
            }
        ).json()
        if r["status"] != 200:
            raise ValueError(f"Cannot get Challenge, {r['message']} [{r['status']}]")
        r = r["data"]

        try:
            challenge = base64.b64decode(r["challenge_b64"])
            license_message = SignedMessage()
            license_message.ParseFromString(challenge)
            if license_message.SerializeToString() != challenge:
                raise DecodeError("partial parse")
        except DecodeError as e:
            raise InvalidLicenseMessage(f"Failed to parse license request, {e}")

        return license_message.SerializeToString()

    def parse_license(self, session_id: bytes, license_message: Union[SignedMessage, bytes, str]) -> None:
        if not license_message:
            raise InvalidLicenseMessage("Cannot parse an empty license_message")

        if isinstance(license_message, str):
            try:
                license_message = base64.b64decode(license_message)
            except (binascii.Error, binascii.Incomplete) as e:
                raise InvalidLicenseMessage(f"Could not decode license_message as Base64, {e}")

        if isinstance(license_message, bytes):
            signed_message = SignedMessage()
            try:
                signed_message.ParseFromString(license_message)
                if signed_message.SerializeToString() != license_message:
                    raise DecodeError("partial parse")
            except DecodeError as e:
                raise InvalidLicenseMessage(f"Could not parse license_message as a SignedMessage, {e}")
            license_message = signed_message

        if not isinstance(license_message, SignedMessage):
            raise InvalidLicenseMessage(f"Expecting license_response to be a SignedMessage, got {license_message!r}")

        if license_message.type != SignedMessage.MessageType.Value("LICENSE"):
            raise InvalidLicenseMessage(
                f"Expecting a LICENSE message, not a "
                f"'{SignedMessage.MessageType.Name(license_message.type)}' message."
            )

        r = self.__session.post(
            url=f"{self.host}/{self.device_name}/parse_license",
            json={
                "session_id": session_id.hex(),
                "license_message": base64.b64encode(license_message.SerializeToString()).decode()
            }
        ).json()
        if r["status"] != 200:
            raise ValueError(f"Cannot parse License, {r['message']} [{r['status']}]")

    def get_keys(self, session_id: bytes, type_: Optional[Union[int, str]] = None) -> list[Key]:
        try:
            if isinstance(type_, str):
                License.KeyContainer.KeyType.Value(type_)  # only test
            elif isinstance(type_, int):
                type_ = License.KeyContainer.KeyType.Name(type_)
            elif type_ is None:
                type_ = "ALL"
            else:
                raise TypeError(f"Expected type_ to be a {License.KeyContainer.KeyType} or int, not {type_!r}")
        except ValueError as e:
            raise ValueError(f"Could not parse type_ as a {License.KeyContainer.KeyType}, {e}")

        r = self.__session.post(
            url=f"{self.host}/{self.device_name}/get_keys/{type_}",
            json={
                "session_id": session_id.hex()
            }
        ).json()
        if r["status"] != 200:
            raise ValueError(f"Could not get {type_} Keys, {r['message']} [{r['status']}]")
        r = r["data"]

        return [
            Key(
                type_=key["type"],
                kid=Key.kid_to_uuid(bytes.fromhex(key["key_id"])),
                key=bytes.fromhex(key["key"]),
                permissions=key["permissions"]
            )
            for key in r["keys"]
        ]


__all__ = ("RemoteCdm",)



from typing import Optional

from Crypto.Random import get_random_bytes

from pywidevine.key import Key
from pywidevine.license_protocol_pb2 import SignedDrmCertificate


class Session:
    def __init__(self, number: int):
        self.number = number
        self.id = get_random_bytes(16)
        self.service_certificate: Optional[SignedDrmCertificate] = None
        self.context: dict[bytes, tuple[bytes, bytes]] = {}
        self.keys: list[Key] = []


__all__ = ("Session",)
